import { CheckBoxControl } from '../../lab/controls/CheckBoxControl.js';
import { CollisionAdvance } from '../../lab/model/CollisionAdvance.js';
import { ContactSim } from '../../lab/engine2D/ContactSim.js';
import { ConstantForceLaw } from '../../lab/model/ConstantForceLaw.js';
import { DampingLaw } from '../../lab/model/DampingLaw.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { Engine2DApp } from './Engine2DApp.js';
import { Force } from '../../lab/model/Force.js';
import { ParameterBoolean, ParameterNumber } from '../../lab/util/Observe.js';
import { JointUtil } from '../../lab/engine2D/JointUtil.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { RotatingTestForce } from './RotatingTestForce.js';
import { Shapes } from '../../lab/engine2D/Shapes.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
export class DoNothingApp extends Engine2DApp {
    constructor(elem_ids) {
        const simRect = new DoubleRect(-6, -6, 6, 6);
        const sim = new ContactSim();
        sim.setExtraAccel("velocity_and_distance");
        const advance = new CollisionAdvance(sim);
        super(elem_ids, simRect, sim, advance);
        this.tightFit = true;
        this.handleForce = 0;
        this.rotateRate = 0.3;
        this.extraBlock = false;
        this.rbo.protoFixedPolygon.setFillStyle('rgb(240,240,240)');
        this.sim.setShowForces(false);
        this.elasticity.setElasticity(0.8);
        this.dampingLaw = new DampingLaw(0.1, 0.15, this.simList);
        this.addPlaybackControls();
        let pb;
        let pn;
        this.addParameter(pb = new ParameterBoolean(this, DoNothingApp.en.TIGHT_FIT, DoNothingApp.i18n.TIGHT_FIT, () => this.getTightFit(), a => this.setTightFit(a)));
        this.addControl(new CheckBoxControl(pb));
        this.addParameter(pb = new ParameterBoolean(this, DoNothingApp.en.EXTRA_BLOCK, DoNothingApp.i18n.EXTRA_BLOCK, () => this.getExtraBlock(), a => this.setExtraBlock(a)));
        this.addControl(new CheckBoxControl(pb));
        this.addParameter(pn = new ParameterNumber(this, DoNothingApp.en.HANDLE_FORCE, DoNothingApp.i18n.HANDLE_FORCE, () => this.getHandleForce(), a => this.setHandleForce(a)));
        this.addControl(new NumericControl(pn));
        this.addParameter(pn = new ParameterNumber(this, DoNothingApp.en.ROTATE_RATE, DoNothingApp.i18n.ROTATE_RATE, () => this.getRotateRate(), a => this.setRotateRate(a)));
        this.addControl(new NumericControl(pn));
        pn = this.dampingLaw.getParameterNumber(DampingLaw.en.DAMPING);
        this.addControl(new NumericControl(pn));
        this.addStandardControls();
        this.makeEasyScript();
        this.addURLScriptButton();
        this.config();
        this.graphSetup();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', dampingLaw: ' + this.dampingLaw.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'DoNothingApp';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('dampingLaw', myName + '.');
        this.terminal.addRegex('DoNothingApp|Engine2DApp', 'sims$engine2D$', false);
    }
    ;
    getSubjects() {
        return super.getSubjects().concat(this.dampingLaw);
    }
    ;
    config() {
        const elasticity = this.elasticity.getElasticity();
        this.sim.cleanSlate();
        this.advance.reset();
        this.sim.addForceLaw(this.dampingLaw);
        this.dampingLaw.connect(this.sim.getSimList());
        DoNothingApp.setup(this.sim, this.tightFit);
        let ds = this.displayList.findShape(DoNothingApp.en.HANDLE);
        ds.setFillStyle('rgba(51,204,255,0.5)');
        ds.setZIndex(2);
        ds = this.displayList.findShape(DoNothingApp.en.SHUTTLE + 1);
        ds.setFillStyle('rgb(200,200,200)');
        ds = this.displayList.findShape(DoNothingApp.en.SHUTTLE + 2);
        ds.setFillStyle('rgb(200,200,200)');
        if (this.extraBlock) {
            const block = Shapes.makeBlock(1, 3, DoNothingApp.en.EXTRA_BLOCK, DoNothingApp.i18n.EXTRA_BLOCK);
            block.setPosition(new Vector(-5.5, -4));
            this.sim.addBody(block);
            this.displayList.findShape(block).setFillStyle('blue');
            this.sim.getBodies().forEach(bod => {
                if (bod.getName().match(/^FIXED.*/) != null) {
                    bod.addNonCollide([block]);
                    block.addNonCollide([bod]);
                }
            });
        }
        if (this.handleForce > 0) {
            const handle = this.sim.getBody('handle');
            let f_law;
            if (this.rotateRate > 0) {
                f_law = new RotatingTestForce(this.sim, handle, new Vector(0, -3), this.handleForce, this.rotateRate);
            }
            else {
                const f = new Force('turning', handle, new Vector(0, -3), 0, new Vector(this.handleForce, 0), 0);
                f_law = new ConstantForceLaw(f);
            }
            this.sim.addForceLaw(f_law);
        }
        this.sim.setElasticity(elasticity);
        this.sim.getVarsList().setTime(0);
        this.sim.saveInitialState();
        this.clock.setTime(0);
        this.clock.setRealTime(0);
        this.easyScript.update();
    }
    ;
    static setup(sim, tightFit) {
        const handle = Shapes.makeBlock(0.8, 6.2, DoNothingApp.en.HANDLE, DoNothingApp.i18n.HANDLE);
        handle.setMass(0.5);
        handle.setDragPoints([new Vector(0, -2.8)]);
        sim.addBody(handle);
        const shuttle_width = tightFit ? 1.0 : 0.98;
        const s1 = Shapes.makeBlock(shuttle_width, 2.5, DoNothingApp.en.SHUTTLE + 1, DoNothingApp.i18n.SHUTTLE + 1);
        s1.setPosition(new Vector(0, 2.0), Math.PI);
        sim.addBody(s1);
        const s2 = Shapes.makeBlock(shuttle_width, 2.5, DoNothingApp.en.SHUTTLE + 2, DoNothingApp.i18n.SHUTTLE + 2);
        s2.setPosition(new Vector(-2.5, 0), Math.PI / 2);
        sim.addBody(s2);
        for (let i = 0; i < 2; i++) {
            for (let j = 0; j < 2; j++) {
                const size = 4;
                const id = j + 2 * i;
                const p = Shapes.makeBlock(size, size, DoNothingApp.en.FIXED_BLOCK + id, DoNothingApp.i18n.FIXED_BLOCK + id);
                const d = 0.507 + size / 2;
                p.setPosition(new Vector(d * (1 - 2 * i), d * (1 - 2 * j)), 0);
                p.setMass(Infinity);
                sim.addBody(p);
                handle.addNonCollide([p]);
                p.addNonCollide([handle]);
            }
        }
        const p1 = s1.getPosition();
        const p2 = s2.getPosition();
        const handleLength = p1.distanceTo(p2);
        const a = Math.atan(-p1.getY() / p2.getX());
        handle.setAngle(-Math.PI / 2 + a);
        JointUtil.attachRigidBody(sim, s1, new Vector(0, 0), handle, new Vector(0, 2.8), 0);
        JointUtil.attachRigidBody(sim, s2, new Vector(0, 0), handle, new Vector(0, 2.8 - handleLength), 0);
        sim.alignConnectors();
    }
    ;
    getHandleForce() {
        return this.handleForce;
    }
    ;
    setHandleForce(value) {
        this.handleForce = value;
        this.config();
        this.broadcastParameter(DoNothingApp.en.HANDLE_FORCE);
    }
    ;
    getRotateRate() {
        return this.rotateRate;
    }
    ;
    setRotateRate(value) {
        this.rotateRate = value;
        this.config();
        this.broadcastParameter(DoNothingApp.en.ROTATE_RATE);
    }
    ;
    getTightFit() {
        return this.tightFit;
    }
    ;
    setTightFit(value) {
        this.tightFit = value;
        this.config();
        this.broadcastParameter(DoNothingApp.en.TIGHT_FIT);
    }
    ;
    getExtraBlock() {
        return this.extraBlock;
    }
    ;
    setExtraBlock(value) {
        this.extraBlock = value;
        this.config();
        this.broadcastParameter(DoNothingApp.en.EXTRA_BLOCK);
    }
    ;
}
DoNothingApp.en = {
    HANDLE_FORCE: 'handle force',
    ROTATE_RATE: 'force rotation rate',
    TIGHT_FIT: 'tight fit',
    EXTRA_BLOCK: 'extra block',
    HANDLE: 'handle',
    SHUTTLE: 'shuttle',
    FIXED_BLOCK: 'fixed block'
};
DoNothingApp.de_strings = {
    HANDLE_FORCE: 'Griff Kraft',
    ROTATE_RATE: 'Kraft, Rotation Tempo',
    TIGHT_FIT: 'exakt passend',
    EXTRA_BLOCK: 'extra Block',
    HANDLE: 'Griff',
    SHUTTLE: 'Shuttle',
    FIXED_BLOCK: 'Festblock'
};
DoNothingApp.i18n = Util.LOCALE === 'de' ? DoNothingApp.de_strings : DoNothingApp.en;
Util.defineGlobal('sims$engine2D$DoNothingApp', DoNothingApp);
