import { DampingLaw } from '../../lab/model/DampingLaw.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { Engine2DApp } from './Engine2DApp.js';
import { GravityLaw } from '../../lab/model/GravityLaw.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { ParameterNumber } from '../../lab/util/Observe.js';
import { Polygon } from '../../lab/engine2D/Polygon.js';
import { RigidBodySim } from '../../lab/engine2D/RigidBodySim.js';
import { Scrim } from '../../lab/engine2D/Scrim.js';
import { Shapes } from '../../lab/engine2D/Shapes.js';
import { SimpleAdvance } from '../../lab/model/SimpleAdvance.js';
import { SixThrusters } from './SixThrusters.js';
import { Spring } from '../../lab/model/Spring.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
export class RigidBodyApp extends Engine2DApp {
    constructor(elem_ids) {
        const simRect = new DoubleRect(-4, -4, 4, 4);
        const sim = new RigidBodySim();
        const advance = new SimpleAdvance(sim);
        super(elem_ids, simRect, sim, advance);
        this.numBods = 5;
        this.mass1 = 1;
        this.thrust = 1.5;
        this.springs_ = [];
        this.restLength = 0.75;
        this.stiffness = 1;
        this.layout.getSimCanvas().setBackground('black');
        this.rbo.protoPolygon.setNameColor('gray');
        this.rbo.protoPolygon.setNameFont('10pt sans-serif');
        this.rbo.protoSpring.setThickness(4);
        this.rbo.protoSpring.setWidth(.4);
        this.elasticity.setElasticity(1.0);
        this.sim.setShowForces(false);
        this.dampingLaw = new DampingLaw(0, 0.1, this.simList);
        this.gravityLaw = new GravityLaw(0, this.simList);
        this.addPlaybackControls();
        let pn = new ParameterNumber(this, RigidBodyApp.en.NUM_BODIES, RigidBodyApp.i18n.NUM_BODIES, () => this.getNumBodies(), a => this.setNumBodies(a));
        pn.setDecimalPlaces(0);
        pn.setLowerLimit(1);
        pn.setUpperLimit(6);
        this.addParameter(pn);
        this.addControl(new NumericControl(pn));
        pn = new ParameterNumber(this, RigidBodyApp.en.THRUST, RigidBodyApp.i18n.THRUST, () => this.getThrust(), a => this.setThrust(a));
        this.addParameter(pn);
        this.addControl(new NumericControl(pn));
        pn = new ParameterNumber(this, RigidBodyApp.en.MASS1, RigidBodyApp.i18n.MASS1, () => this.getMass1(), a => this.setMass1(a));
        this.addParameter(pn);
        this.addControl(new NumericControl(pn));
        pn = this.gravityLaw.getParameterNumber(GravityLaw.en.GRAVITY);
        this.addControl(new NumericControl(pn));
        this.watchEnergyChange(pn);
        pn = this.dampingLaw.getParameterNumber(DampingLaw.en.DAMPING);
        this.addControl(new NumericControl(pn));
        pn = new ParameterNumber(this, RigidBodyApp.en.SPRING_STIFFNESS, RigidBodyApp.i18n.SPRING_STIFFNESS, () => this.getStiffness(), a => this.setStiffness(a));
        this.addParameter(pn);
        this.addControl(new NumericControl(pn));
        pn = new ParameterNumber(this, RigidBodyApp.en.SPRING_LENGTH, RigidBodyApp.i18n.SPRING_LENGTH, () => this.getRestLength(), a => this.setRestLength(a));
        this.addParameter(pn);
        this.addControl(new NumericControl(pn));
        this.addStandardControls();
        this.makeEasyScript();
        this.addURLScriptButton();
        this.config();
        this.graphSetup();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', dampingLaw: ' + this.dampingLaw.toStringShort()
            + ', gravityLaw: ' + this.gravityLaw.toStringShort()
            + super.toString();
    }
    ;
    addSpring(s) {
        this.springs_.push(s);
        this.sim.addForceLaw(s);
        this.simList.add(s);
    }
    ;
    getClassName() {
        return 'RigidBodyApp';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('gravityLaw|dampingLaw', myName + '.');
        this.terminal.addRegex('RigidBodyApp|Engine2DApp', 'sims$engine2D$', false);
    }
    ;
    getSubjects() {
        return super.getSubjects().concat(this.dampingLaw, this.gravityLaw);
    }
    ;
    static makeBlock(num) {
        return Shapes.makeBlock(1, 3, RigidBodyApp.en.BLOCK + num, RigidBodyApp.i18n.BLOCK + num);
    }
    ;
    config() {
        const elasticity = this.elasticity.getElasticity();
        this.sim.cleanSlate();
        Polygon.ID = 1;
        this.springs_ = [];
        this.advance.reset();
        this.sim.addForceLaw(this.dampingLaw);
        this.dampingLaw.connect(this.sim.getSimList());
        this.sim.addForceLaw(this.gravityLaw);
        this.gravityLaw.connect(this.sim.getSimList());
        make_blocks: if (this.numBods >= 1) {
            const p1 = RigidBodyApp.makeBlock(1);
            p1.setPosition(new Vector(-3.3, 0), 0);
            p1.setVelocity(new Vector(0.3858, -0.3608), -0.3956);
            p1.setMass(this.mass1);
            this.sim.addBody(p1);
            this.thrust2 = SixThrusters.make(this.thrust, p1);
            this.rbeh.setThrusters(this.thrust2, 'left');
            this.sim.addForceLaw(this.thrust2);
            this.addSpring(new Spring('spring 1', Scrim.getScrim(), new Vector(-2, -2), p1, new Vector(0.15, 0.7), this.restLength, this.stiffness));
            this.displayList.findShape(p1).setFillStyle('cyan');
            if (this.numBods < 2) {
                break make_blocks;
            }
            const p2 = RigidBodyApp.makeBlock(2);
            p2.setPosition(new Vector(-2.2, 0), 0);
            p2.setVelocity(new Vector(0.26993, -0.01696), -0.30647);
            this.sim.addBody(p2);
            this.thrust1 = SixThrusters.make(this.thrust, p2);
            this.rbeh.setThrusters(this.thrust1, 'right');
            this.sim.addForceLaw(this.thrust1);
            this.addSpring(new Spring('spring 2', Scrim.getScrim(), new Vector(2, 2), p2, new Vector(0.15, 0.7), this.restLength, this.stiffness));
            this.addSpring(new Spring('spring 3', p2, new Vector(0.15, -0.7), p1, new Vector(0.15, -0.7), this.restLength, this.stiffness));
            this.displayList.findShape(p2).setFillStyle('orange');
            if (this.numBods < 3) {
                break make_blocks;
            }
            const p3 = RigidBodyApp.makeBlock(3);
            p3.setPosition(new Vector(2.867, -0.113), 0);
            p3.setVelocity(new Vector(-0.29445, -0.11189), -0.23464);
            this.sim.addBody(p3);
            this.addSpring(new Spring('spring 4', p3, new Vector(0.15, 0.7), p2, new Vector(0.15, -0.7), this.restLength, this.stiffness));
            this.displayList.findShape(p3).setFillStyle('#9f3');
            if (this.numBods < 4) {
                break make_blocks;
            }
            const p4 = RigidBodyApp.makeBlock(4);
            p4.setPosition(new Vector(1.36, 2.5), -Math.PI / 4);
            p4.setVelocity(new Vector(-0.45535, -0.37665), 0.36526);
            this.sim.addBody(p4);
            this.addSpring(new Spring('spring 5', p4, new Vector(0.15, 0.7), p3, new Vector(0.15, -0.7), this.restLength, this.stiffness));
            this.displayList.findShape(p4).setFillStyle('#f6c');
            if (this.numBods < 5) {
                break make_blocks;
            }
            const p5 = RigidBodyApp.makeBlock(5);
            p5.setPosition(new Vector(-2, 2.5), Math.PI / 2 + 0.1);
            this.sim.addBody(p5);
            this.addSpring(new Spring('spring 6', p5, new Vector(0.15, 0.7), p4, new Vector(0.15, -0.7), this.restLength, this.stiffness));
            this.displayList.findShape(p5).setFillStyle('#39f');
            if (this.numBods >= 6) {
                break make_blocks;
            }
            const p6 = RigidBodyApp.makeBlock(6);
            p6.setPosition(new Vector(0.08, 0.127), 0.888);
            this.sim.addBody(p6);
            this.addSpring(new Spring('spring 7', p6, new Vector(0.15, 0.7), p5, new Vector(0.15, -0.7), this.restLength, this.stiffness));
            this.displayList.findShape(p6).setFillStyle('#c99');
        }
        this.sim.getVarsList().setTime(0);
        this.sim.saveInitialState();
        this.clock.setTime(0);
        this.clock.setRealTime(0);
        this.easyScript.update();
    }
    ;
    getMass1() {
        return this.mass1;
    }
    ;
    setMass1(value) {
        if (this.mass1 != value) {
            this.mass1 = value;
            const body1 = this.sim.getBody(RigidBodyApp.en.BLOCK + '1');
            body1.setMass(value);
            this.broadcastParameter(RigidBodyApp.en.MASS1);
        }
    }
    ;
    getNumBodies() {
        return this.numBods;
    }
    ;
    setNumBodies(value) {
        this.numBods = value;
        this.config();
        this.broadcastParameter(RigidBodyApp.en.NUM_BODIES);
    }
    ;
    getStiffness() {
        return this.stiffness;
    }
    ;
    setStiffness(value) {
        this.stiffness = value;
        this.springs_.forEach(s => s.setStiffness(value));
        this.sim.getVarsList().incrSequence(2, 3);
        this.broadcastParameter(RigidBodyApp.en.SPRING_STIFFNESS);
    }
    ;
    getRestLength() {
        return this.restLength;
    }
    ;
    setRestLength(value) {
        this.restLength = value;
        this.springs_.forEach(s => s.setRestLength(value));
        this.sim.getVarsList().incrSequence(2, 3);
        this.broadcastParameter(RigidBodyApp.en.SPRING_LENGTH);
    }
    ;
    getThrust() {
        return this.thrust;
    }
    ;
    setThrust(value) {
        this.thrust = value;
        this.thrust1.setMagnitude(value);
        this.thrust2.setMagnitude(value);
        this.broadcastParameter(RigidBodyApp.en.THRUST);
    }
    ;
}
RigidBodyApp.en = {
    NUM_BODIES: 'number of objects',
    SPRING_LENGTH: 'spring length',
    SPRING_STIFFNESS: 'spring stiffness',
    THRUST: 'thrust',
    BLOCK: 'block',
    MASS1: 'mass of block1'
};
RigidBodyApp.de_strings = {
    NUM_BODIES: 'Anzahl von Objekten',
    SPRING_LENGTH: 'Federlänge',
    SPRING_STIFFNESS: 'Federsteifheit',
    THRUST: 'Schubkraft',
    BLOCK: 'Block',
    MASS1: 'Masse von Block1'
};
RigidBodyApp.i18n = Util.LOCALE === 'de' ? RigidBodyApp.de_strings : RigidBodyApp.en;
Util.defineGlobal('sims$engine2D$RigidBodyApp', RigidBodyApp);
