import { Clock } from '../../lab/util/Clock.js';
import { DisplayText } from '../../lab/view/DisplayText.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { LabCanvas } from '../../lab/view/LabCanvas.js';
import { SimView } from '../../lab/view/SimView.js';
import { Timer } from '../../lab/util/Timer.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
export function makeBikeTimerApp(sound_url) {
    Util.setErrorHandler();
    const simDiv = Util.getElementById('sim_applet');
    const canvas = document.createElement('canvas');
    const simCanvas = new LabCanvas(canvas, 'canvas1');
    simCanvas.setSize(800, 500);
    simDiv.appendChild(canvas);
    const simRect = new DoubleRect(-5, -5, 5, 5);
    const simView = new SimView('simView', simRect);
    const timer = new Timer();
    const clock = new Clock();
    const dtext1 = new DisplayText('0.0', new Vector(2, -1));
    dtext1.setFont('160pt sans-serif');
    dtext1.setTextAlign('center');
    dtext1.setTextBaseline('middle');
    simView.getDisplayList().add(dtext1);
    simCanvas.addView(simView);
    const beep2 = new Audio(sound_url);
    if (0 == 1) {
        beep2.play();
    }
    let period = 10.1;
    let startTime = 3.0;
    let shouldBeep = true;
    clock.setTime(-startTime);
    timer.setCallBack(() => {
        const now = clock.getTime();
        const t = period * Math.ceil(now / period) - now;
        dtext1.setText(t.toFixed(1));
        if (t > period - 2) {
            if (shouldBeep) {
                beep2.play();
                shouldBeep = false;
            }
        }
        else {
            shouldBeep = true;
        }
        simCanvas.paint();
    });
    const resetButton = Util.getElementById('reset_button');
    resetButton.onclick = () => {
        clock.pause();
        clock.setTime(-startTime);
        shouldBeep = true;
    };
    const stopButton = Util.getElementById('stop_button');
    stopButton.onclick = () => clock.pause();
    const startButton = Util.getElementById('start_button');
    startButton.onclick = () => clock.resume();
    const field_period = Util.getElementById('period_field');
    field_period.style.textAlign = 'right';
    field_period.value = period.toFixed(1);
    field_period.addEventListener('change', _event => {
        const value = parseFloat(field_period.value);
        if (isNaN(value)) {
            field_period.value = period.toFixed(1);
        }
        else {
            period = value;
        }
    });
    const field_start = Util.getElementById('start_field');
    field_start.style.textAlign = 'right';
    field_start.value = startTime.toFixed(1);
    field_start.addEventListener('change', _event => {
        const value = parseFloat(field_start.value);
        if (isNaN(value)) {
            field_start.value = startTime.toFixed(1);
        }
        else {
            startTime = value;
        }
    });
    timer.startFiring();
    clock.pause();
}
;
Util.defineGlobal('sims$experimental$makeBikeTimerApp', makeBikeTimerApp);
