import { AbstractStringShape } from './AbstractStringShape.js';
import { Util } from '../../lab/util/Util.js';
export class FlatShape extends AbstractStringShape {
    constructor(length) {
        super(length, StringShapes.en.FLAT, StringShapes.i18n.FLAT);
    }
    ;
    getClassName() {
        return 'FlatShape';
    }
    ;
    position(_x) {
        return 0;
    }
    ;
    velocity(_x) {
        return 0;
    }
    ;
}
Util.defineGlobal('sims$pde$FlatShape', FlatShape);
export class SquarePulseShape extends AbstractStringShape {
    constructor(length, pulseWidth) {
        super(length, StringShapes.en.SQUARE_PULSE, StringShapes.i18n.SQUARE_PULSE);
        this.w_ = pulseWidth || length / 8;
    }
    ;
    getClassName() {
        return 'SquarePulseShape';
    }
    ;
    position(x) {
        const middle = this.length_ / 2;
        return (x >= middle - this.w_ && x <= middle + this.w_) ? 0.1 : 0.0;
    }
    ;
    velocity(_x) {
        return 0;
    }
    ;
}
Util.defineGlobal('sims$pde$SquarePulseShape', SquarePulseShape);
export class TrianglePulseShape extends AbstractStringShape {
    constructor(length) {
        super(length, StringShapes.en.TRIANGLE_PULSE, StringShapes.i18n.TRIANGLE_PULSE);
        this.w_ = length / 8;
    }
    ;
    getClassName() {
        return 'TrianglePulseShape';
    }
    ;
    position(x) {
        x = x - this.w_;
        if (x < -this.w_ || x > this.w_) {
            return 0;
        }
        else {
            return 0.1 * ((x < 0) ? x / this.w_ + 1 : -x / this.w_ + 1);
        }
    }
    ;
    velocity(x) {
        x = x - this.w_;
        if (x < -this.w_ || x > this.w_) {
            return 0;
        }
        else {
            return -(0.1 / this.w_) * ((x < 0) ? 1 : -1);
        }
    }
    ;
}
Util.defineGlobal('sims$pde$TrianglePulseShape', TrianglePulseShape);
export class SinePulseShape extends AbstractStringShape {
    constructor(length) {
        super(length, StringShapes.en.SINE_PULSE, StringShapes.i18n.SINE_PULSE);
        this.w_ = length / 8;
    }
    ;
    getClassName() {
        return 'SinePulseShape';
    }
    ;
    position(x) {
        x = x - this.w_;
        if (x < -this.w_ || x > this.w_) {
            return 0;
        }
        else {
            return 0.05 * Math.sin(Math.PI * x / this.w_);
        }
    }
    ;
    velocity(x) {
        x = x - this.w_;
        if (x < -this.w_ || x > this.w_) {
            return 0;
        }
        else {
            return -0.05 * (Math.PI / this.w_) * Math.cos(Math.PI * x / this.w_);
        }
    }
    ;
}
Util.defineGlobal('sims$pde$SinePulseShape', SinePulseShape);
export class HalfSinePulseShape extends AbstractStringShape {
    constructor(length) {
        super(length, StringShapes.en.HALF_SINE_PULSE, StringShapes.i18n.HALF_SINE_PULSE);
        this.w_ = length / 3;
    }
    ;
    getClassName() {
        return 'HalfSinePulseShape';
    }
    ;
    position(x) {
        if (x > this.w_)
            return 0;
        else
            return 0.05 * Math.sin(Math.PI * x / this.w_);
    }
    ;
    velocity(_x) {
        return 0;
    }
    ;
}
Util.defineGlobal('sims$pde$HalfSinePulseShape', HalfSinePulseShape);
export class MultiSineShape extends AbstractStringShape {
    constructor(length) {
        super(length, StringShapes.en.MULTI_SINE, StringShapes.i18n.MULTI_SINE);
    }
    ;
    getClassName() {
        return 'MultiSineShape';
    }
    ;
    position(x) {
        return 0.1 * (Math.sin(2 * Math.PI * x / this.length_)
            + Math.sin(4 * Math.PI * x / this.length_)
            + Math.sin(6 * Math.PI * x / this.length_)) / 3;
    }
    ;
    velocity(_x) {
        return 0;
    }
    ;
}
Util.defineGlobal('sims$pde$MultiSineShape', MultiSineShape);
export class TriangleShape extends AbstractStringShape {
    constructor(length) {
        super(length, StringShapes.en.TRIANGLE, StringShapes.i18n.TRIANGLE);
    }
    ;
    getClassName() {
        return 'TriangleShape';
    }
    ;
    position(x) {
        return 0.2 * (x < this.length_ / 2 ? x / this.length_ : 1 - x / this.length_);
    }
    ;
    velocity(_x) {
        return 0;
    }
    ;
}
Util.defineGlobal('sims$pde$TriangleShape', TriangleShape);
class StringShapes {
    constructor() {
        throw '';
    }
    ;
}
StringShapes.en = {
    FLAT: 'Flat',
    HALF_SINE_PULSE: 'Half Sine Pulse',
    MULTI_SINE: 'Multiple Sine',
    SINE_PULSE: 'Sine Pulse',
    SQUARE_PULSE: 'Square Pulse',
    TRIANGLE: 'Triangle',
    TRIANGLE_PULSE: 'Triangle Pulse'
};
StringShapes.de_strings = {
    FLAT: 'Flach',
    MULTI_SINE: 'Merhfach Sine',
    HALF_SINE_PULSE: 'Halbes Sine Puls',
    SINE_PULSE: 'Sine Pulse',
    SQUARE_PULSE: 'Quadrat Puls',
    TRIANGLE: 'Dreieck',
    TRIANGLE_PULSE: 'Dreieck Puls'
};
StringShapes.i18n = Util.LOCALE === 'de' ? StringShapes.de_strings : StringShapes.en;
Util.defineGlobal('sims$pde$StringShapes', StringShapes);
