import { AbstractSimObject } from '../../lab/model/SimObject.js';
import { AbstractSubject } from '../../lab/util/AbstractSubject.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { EnergyInfo } from '../../lab/model/EnergySystem.js';
import { GenericEvent } from '../../lab/util/Observe.js';
import { ParameterNumber } from '../../lab/util/Observe.js';
import { PointMass } from '../../lab/model/PointMass.js';
import { SimList } from '../../lab/model/SimList.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
export class StringSim extends AbstractSubject {
    constructor(shape, opt_simList) {
        super('SIM');
        this.length_ = 5;
        this.damping_ = 0;
        this.density_ = 1;
        this.tension_ = 10;
        this.block_ = PointMass.makeRectangle(0.7, 0.05, 'block');
        this.startPosition_ = new Vector(-this.block_.getWidth() / 2, 0);
        this.w1_ = [];
        this.w2_ = [];
        this.w3_ = [];
        this.wIdx_ = 1;
        this.w_ = this.w1_;
        this.deltaX_ = 0.1;
        this.deltaT_ = 0.0025;
        this.avgLen_ = 10;
        this.times_ = Util.newNumberArray(this.avgLen_);
        this.stab_ = Util.newNumberArray(this.avgLen_);
        this.sequence_ = 0;
        this.timeIdx_ = 0;
        this.lastTime_ = -100;
        this.nowTime_ = 0;
        this.numPoints_ = 1001;
        this.potentialOffset_ = 0;
        this.simList_ = opt_simList || new SimList();
        this.block_.setPosition(this.startPosition_);
        this.simList_.add(this.block_);
        this.shape_ = shape;
        this.curve_ = new StringPath(this);
        this.simList_.add(this.curve_);
        let pn;
        this.addParameter(new ParameterNumber(this, StringSim.en.DAMPING, StringSim.i18n.DAMPING, () => this.getDamping(), a => this.setDamping(a)));
        this.addParameter(new ParameterNumber(this, StringSim.en.DENSITY, StringSim.i18n.DENSITY, () => this.getDensity(), a => this.setDensity(a)));
        this.addParameter(new ParameterNumber(this, StringSim.en.TENSION, StringSim.i18n.TENSION, () => this.getTension(), a => this.setTension(a)));
        this.addParameter(new ParameterNumber(this, StringSim.en.NUM_POINTS, StringSim.i18n.NUM_POINTS, () => this.getNumPoints(), a => this.setNumPoints(a)));
        this.addParameter(new ParameterNumber(this, StringSim.en.TIME_STEP, StringSim.i18n.TIME_STEP, () => this.getTimeStep(), a => this.setTimeStep(a)));
        this.addParameter(pn = new ParameterNumber(this, EnergyInfo.en.PE_OFFSET, EnergyInfo.i18n.PE_OFFSET, () => this.getPEOffset(), a => this.setPEOffset(a)));
        pn.setLowerLimit(Number.NEGATIVE_INFINITY);
        pn.setSignifDigits(5);
        this.initializeFromShape();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', density_: ' + Util.NF(this.density_)
            + ', tension_: ' + this.tension_
            + ', potentialOffset_: ' + Util.NF(this.potentialOffset_)
            + super.toString();
    }
    ;
    getClassName() {
        return 'StringSim';
    }
    ;
    getLength() {
        return this.length_;
    }
    ;
    getSimList() {
        return this.simList_;
    }
    ;
    getTime() {
        return this.nowTime_;
    }
    ;
    modifyObjects() {
    }
    ;
    saveInitialState() {
    }
    ;
    reset() {
        this.nowTime_ = 0;
        this.initializeFromShape();
        this.simList_.removeTemporary(Infinity);
        this.modifyObjects();
        this.broadcast(new GenericEvent(this, 'RESET'));
    }
    ;
    startDrag(simObject, _location, _offset, _dragBody, _modifiers) {
        return simObject == this.block_;
    }
    ;
    mouseDrag(simObject, location, offset) {
        if (simObject == this.block_) {
            const p = location.subtract(offset);
            this.block_.setPosition(new Vector(this.block_.getPosition().getX(), p.getY()));
        }
    }
    ;
    finishDrag(_simObject, _location, _offset) {
    }
    ;
    handleKeyEvent(_evt, _pressed, _modifiers) {
    }
    ;
    getPoint(idx, point) {
        if (idx < 0 || idx > this.w_.length) {
            throw '';
        }
        point.setTo(idx * this.length_ / this.numPoints_, this.w_[idx]);
    }
    ;
    initializeFromShape() {
        this.block_.setPosition(this.startPosition_);
        this.length_ = this.shape_.getLength();
        this.deltaX_ = this.length_ / (this.numPoints_ - 1);
        this.w1_ = Util.newNumberArray(this.numPoints_);
        this.w2_ = Util.newNumberArray(this.numPoints_);
        this.w3_ = Util.newNumberArray(this.numPoints_);
        this.wIdx_ = 2;
        this.w_ = this.w2_;
        const r = (this.deltaT_ * this.deltaT_ * this.tension_ / this.density_) /
            (this.deltaX_ * this.deltaX_);
        this.w1_[0] = this.w1_[this.numPoints_ - 1] = 0;
        this.w2_[0] = this.w2_[this.numPoints_ - 1] = 0;
        for (let i = 1; i < this.numPoints_ - 1; i++) {
            this.w1_[i] = this.shape_.position(i * this.deltaX_);
            this.w2_[i] = (1 - r) * this.shape_.position(i * this.deltaX_)
                + (r / 2) * (this.shape_.position((i + 1) * this.deltaX_)
                    + this.shape_.position((i - 1) * this.deltaX_));
            this.w2_[i] += this.deltaT_ * Math.sqrt(this.tension_ / this.density_)
                * this.shape_.velocity(i * this.deltaX_);
        }
    }
    ;
    advance() {
        let wNew;
        let w;
        let wOld;
        switch (this.wIdx_) {
            case 1:
                w = this.w1_;
                wOld = this.w3_;
                wNew = this.w2_;
                this.wIdx_ = 2;
                break;
            case 2:
                w = this.w2_;
                wOld = this.w1_;
                wNew = this.w3_;
                this.wIdx_ = 3;
                break;
            case 3:
                w = this.w3_;
                wOld = this.w2_;
                wNew = this.w1_;
                this.wIdx_ = 1;
                break;
            default:
                throw '';
        }
        const N = this.numPoints_ - 1;
        wNew[0] = 0;
        wNew[N] = 0;
        wNew[0] = this.block_.getPosition().getY();
        const r = (this.deltaT_ * this.deltaT_ * this.tension_ / this.density_) /
            (this.deltaX_ * this.deltaX_);
        for (let i = 1; i <= N - 1; i++) {
            wNew[i] = 2 * (1 - r) * w[i] + r * (w[i + 1] + w[i - 1]) - wOld[i];
            if (this.damping_ > 0) {
                wNew[i] += -this.damping_ * (w[i] - wOld[i]) * this.deltaT_ / this.density_;
            }
        }
        this.nowTime_ += this.deltaT_;
        this.w_ = wNew;
        this.sequence_++;
        if (this.sequence_ >= Util.MAX_INTEGER) {
            this.sequence_ = 0;
        }
    }
    ;
    getEnergyInfo() {
        let wNew;
        let w;
        switch (this.wIdx_) {
            case 1:
                wNew = this.w1_;
                w = this.w3_;
                break;
            case 2:
                wNew = this.w2_;
                w = this.w1_;
                break;
            case 3:
                wNew = this.w3_;
                w = this.w2_;
                break;
            default:
                throw '';
        }
        let ke = 0;
        let pe = 0;
        for (let i = 1; i < this.numPoints_ - 1; i++) {
            let diff = (wNew[i - 1] - wNew[i + 1]) / (2 * this.deltaX_);
            pe += diff * diff * this.deltaX_;
            diff = (wNew[i] - w[i]) / this.deltaT_;
            ke += diff * diff * this.deltaX_;
        }
        ke = 0.5 * this.density_ * ke;
        pe = 0.5 * this.tension_ * pe;
        return new EnergyInfo(pe + this.potentialOffset_, ke);
    }
    ;
    getPEOffset() {
        return this.potentialOffset_;
    }
    setPEOffset(value) {
        this.potentialOffset_ = value;
        this.broadcastParameter(EnergyInfo.en.PE_OFFSET);
    }
    ;
    getSequence() {
        return this.sequence_;
    }
    ;
    getShape() {
        return this.shape_;
    }
    ;
    setShape(shape) {
        this.shape_ = shape;
        this.initializeFromShape();
    }
    ;
    getNumPoints() {
        return this.numPoints_;
    }
    ;
    setNumPoints(value) {
        if (value != this.numPoints_) {
            this.numPoints_ = value;
            this.reset();
            this.broadcastParameter(StringSim.en.NUM_POINTS);
        }
    }
    ;
    getTimeStep() {
        return this.deltaT_;
    }
    ;
    setTimeStep(value) {
        if (value != this.deltaT_) {
            this.deltaT_ = value;
            this.reset();
            this.broadcastParameter(StringSim.en.TIME_STEP);
        }
    }
    ;
    getDamping() {
        return this.damping_;
    }
    ;
    setDamping(value) {
        this.damping_ = value;
        this.broadcastParameter(StringSim.en.DAMPING);
    }
    ;
    getDensity() {
        return this.density_;
    }
    ;
    setDensity(value) {
        this.density_ = value;
        this.broadcastParameter(StringSim.en.DENSITY);
    }
    ;
    getTension() {
        return this.tension_;
    }
    ;
    setTension(value) {
        this.tension_ = value;
        this.broadcastParameter(StringSim.en.TENSION);
    }
    ;
    getStability() {
        return Math.sqrt(this.tension_ / this.density_) * this.deltaT_ / this.deltaX_;
    }
    ;
}
StringSim.en = {
    DAMPING: 'damping',
    DENSITY: 'density',
    TENSION: 'tension',
    SHAPE: 'shape',
    NUM_POINTS: 'number of points',
    TIME_STEP: 'time step'
};
StringSim.de_strings = {
    DAMPING: 'Dämpfung',
    DENSITY: 'Dichte',
    TENSION: 'Spannung',
    SHAPE: 'Form',
    NUM_POINTS: 'Anzahl von Punkten',
    TIME_STEP: 'Zeitschritt'
};
StringSim.i18n = Util.LOCALE === 'de' ? StringSim.de_strings : StringSim.en;
Util.defineGlobal('sims$pde$StringSim', StringSim);
export class StringPath extends AbstractSimObject {
    constructor(sim) {
        super('string');
        this.sim_ = sim;
    }
    ;
    toString() {
        return super.toString().slice(0, -1)
            + ', sim: ' + this.sim_.toStringShort()
            + '}';
    }
    ;
    getClassName() {
        return 'StringPath';
    }
    ;
    getBoundsWorld() {
        const len = this.sim_.getLength();
        const height = 1;
        return new DoubleRect(0, -height, len, height);
    }
    ;
    getIterator(_numPoints) {
        return new StringIterator(this.sim_);
    }
    ;
    getSequence() {
        return this.sim_.getSequence();
    }
    ;
}
Util.defineGlobal('sims$pde$StringPath', StringPath);
export class StringIterator {
    constructor(sim) {
        this.idx_ = -1;
        this.sim_ = sim;
    }
    ;
    nextPoint(point) {
        const n = this.sim_.getNumPoints();
        if (this.idx_ >= n - 1) {
            return false;
        }
        this.idx_++;
        this.sim_.getPoint(this.idx_, point);
        return true;
    }
    ;
}
Util.defineGlobal('sims$pde$StringIterator', StringIterator);
