import { AbstractApp } from '../common/AbstractApp.js';
import { CartPendulumSim } from './CartPendulumSim.js';
import { ConcreteLine } from '../../lab/model/ConcreteLine.js';
import { DisplayLine } from '../../lab/view/DisplayLine.js';
import { DisplayShape } from '../../lab/view/DisplayShape.js';
import { DisplaySpring } from '../../lab/view/DisplaySpring.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { SimpleAdvance } from '../../lab/model/SimpleAdvance.js';
import { SliderControl } from '../../lab/controls/SliderControl.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
export class CartPendulumApp extends AbstractApp {
    constructor(elem_ids) {
        Util.setErrorHandler();
        const simRect = new DoubleRect(-3, -4, 3, 2);
        const sim = new CartPendulumSim();
        const advance = new SimpleAdvance(sim);
        super(elem_ids, simRect, sim, advance, sim, sim);
        this.sim.getVarsList().setValue(0, 1.0);
        sim.initWork();
        this.track = new ConcreteLine('track', new Vector(-3, -0.15), new Vector(3, -0.15));
        let dl = new DisplayLine(this.track).setColor('lightGray');
        dl.setThickness(1);
        this.displayList.add(dl);
        this.cart = new DisplayShape(this.simList.getPointMass('cart'));
        this.cart.setStrokeStyle('');
        this.cart.setFillStyle('lightGray');
        this.displayList.add(this.cart);
        this.bob = new DisplayShape(this.simList.getPointMass('bob'));
        this.bob.setStrokeStyle('');
        this.bob.setFillStyle('blue');
        this.displayList.add(this.bob);
        this.rod = new DisplayLine(this.simList.getConcreteLine('rod'));
        this.displayList.add(this.rod);
        this.spring = new DisplaySpring(this.simList.getSpring('spring'));
        this.spring.setWidth(0.3);
        this.displayList.add(this.spring);
        sim.modifyObjects();
        this.addPlaybackControls();
        let pn = sim.getParameterNumber(CartPendulumSim.en.GRAVITY);
        this.addControl(new SliderControl(pn, 0, 20, false));
        pn = sim.getParameterNumber(CartPendulumSim.en.CART_MASS);
        this.addControl(new SliderControl(pn, 0.2, 20.2, true));
        pn = sim.getParameterNumber(CartPendulumSim.en.PENDULUM_MASS);
        this.addControl(new SliderControl(pn, 0.2, 20.2, true));
        pn = sim.getParameterNumber(CartPendulumSim.en.CART_DAMPING);
        this.addControl(new SliderControl(pn, 0, 20, false));
        pn = sim.getParameterNumber(CartPendulumSim.en.PENDULUM_DAMPING);
        this.addControl(new SliderControl(pn, 0, 20, false));
        pn = sim.getParameterNumber(CartPendulumSim.en.PENDULUM_LENGTH);
        this.addControl(new SliderControl(pn, 0.1, 10.1, true));
        pn = sim.getParameterNumber(CartPendulumSim.en.SPRING_STIFFNESS);
        this.addControl(new SliderControl(pn, 0.1, 100.1, true));
        this.addStandardControls();
        this.makeEasyScript();
        this.addURLScriptButton();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', cart: ' + this.cart.toStringShort()
            + ', bob: ' + this.bob.toStringShort()
            + ', rod: ' + this.rod.toStringShort()
            + ', spring: ' + this.spring.toStringShort()
            + ', track: ' + this.track.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'CartPendulumApp';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('rod|cart|bob|spring|track', myName + '.');
    }
    ;
}
Util.defineGlobal('sims$pendulum$CartPendulumApp', CartPendulumApp);
