import { AbstractApp } from '../common/AbstractApp.js';
import { DisplayLine } from '../../lab/view/DisplayLine.js';
import { DisplayShape } from '../../lab/view/DisplayShape.js';
import { DoublePendulumSim } from './DoublePendulumSim.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { PointMass } from '../../lab/model/PointMass.js';
import { SimpleAdvance } from '../../lab/model/SimpleAdvance.js';
import { SliderControl } from '../../lab/controls/SliderControl.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
export class DoublePendulumApp extends AbstractApp {
    constructor(elem_ids) {
        Util.setErrorHandler();
        const simRect = new DoubleRect(-2, -2.2, 2, 1.5);
        const sim = new DoublePendulumSim();
        const advance = new SimpleAdvance(sim);
        super(elem_ids, simRect, sim, advance, sim, sim);
        this.rod1 = new DisplayLine(this.simList.getConcreteLine('rod1'));
        this.displayList.add(this.rod1);
        this.rod2 = new DisplayLine(this.simList.getConcreteLine('rod2'));
        this.displayList.add(this.rod2);
        this.bob1 = new DisplayShape(this.simList.getPointMass('bob1'));
        this.bob1.setFillStyle('blue');
        this.displayList.add(this.bob1);
        this.bob2 = new DisplayShape(this.simList.getPointMass('bob2'));
        this.bob2.setFillStyle('blue');
        this.displayList.add(this.bob2);
        if (1 == 1) {
            this.marker1 = PointMass.makeCircle(0.2, 'marker1');
            this.marker1.setPosition(new Vector(-3, 1));
            this.marker1Shape = new DisplayShape(this.marker1);
            this.marker1Shape.setFillStyle('');
            this.marker1Shape.setStrokeStyle('red');
            this.marker1Shape.setDragable(true);
            this.displayList.add(this.marker1Shape);
        }
        this.graph.line.setXVariable(0);
        this.graph.line.setYVariable(2);
        if (0 == 1) {
            this.graph.line.xTransform = function (x, y) { return y * Math.cos(x); };
            this.graph.line.yTransform = function (x, y) { return y * Math.sin(x); };
        }
        this.addPlaybackControls();
        let pn;
        pn = sim.getParameterNumber(DoublePendulumSim.en.ROD_1_LENGTH);
        this.addControl(new SliderControl(pn, 0.1, 10.1, true));
        pn = sim.getParameterNumber(DoublePendulumSim.en.ROD_2_LENGTH);
        this.addControl(new SliderControl(pn, 0.1, 10.1, true));
        pn = sim.getParameterNumber(DoublePendulumSim.en.MASS_1);
        this.addControl(new SliderControl(pn, 0.2, 20.2, true));
        pn = sim.getParameterNumber(DoublePendulumSim.en.MASS_2);
        this.addControl(new SliderControl(pn, 0.2, 20.2, true));
        pn = sim.getParameterNumber(DoublePendulumSim.en.GRAVITY);
        this.addControl(new SliderControl(pn, 0, 20, false));
        this.addStandardControls();
        this.makeEasyScript();
        this.addURLScriptButton();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', bob1: ' + this.bob1.toStringShort()
            + ', bob2: ' + this.bob2.toStringShort()
            + ', rod1: ' + this.rod2.toStringShort()
            + ', rod2: ' + this.rod2.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'DoublePendulumApp';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('rod1|rod2|bob1|bob2', myName + '.');
        this.terminal.addRegex('DoublePendulumSim', 'sims$pendulum$', false);
    }
    ;
}
Util.defineGlobal('sims$pendulum$DoublePendulumApp', DoublePendulumApp);
