import { AbstractODESim } from '../../lab/model/ODESim.js';
import { ConcreteLine } from '../../lab/model/ConcreteLine.js';
import { EnergyInfo } from '../../lab/model/EnergySystem.js';
import { ParameterNumber } from '../../lab/util/Observe.js';
import { PointMass } from '../../lab/model/PointMass.js';
import { Spring } from '../../lab/model/Spring.js';
import { Util } from '../../lab/util/Util.js';
import { VarsList } from '../../lab/model/VarsList.js';
import { Vector } from '../../lab/util/Vector.js';
const W1 = 0;
const W1P = 1;
const W2 = 2;
const W2P = 3;
const TIME = 4;
const TOPX = 5;
const TOPXP = 6;
const TOPY = 7;
const TOPYP = 8;
const KE = 9;
const PE = 10;
const TE = 11;
export class MoveableDoublePendulumSim extends AbstractODESim {
    constructor(opt_name) {
        super(opt_name);
        this.length1_ = 1;
        this.length2_ = 1;
        this.gravity_ = 10;
        this.damping_ = 0.5;
        this.pendulumDragging_ = false;
        this.springDragging_ = false;
        this.anchorDamping_ = 0.8;
        this.springStiffness_ = 3;
        this.frequency_ = 30;
        this.amplitude_ = 200;
        this.potentialOffset_ = 0;
        this.running_ = false;
        this.anchor_ = PointMass.makeSquare(0.3, 'anchor');
        this.rod1_ = new ConcreteLine('rod1');
        this.rod2_ = new ConcreteLine('rod2');
        this.bob1_ = PointMass.makeCircle(0.2, 'bob1');
        this.bob2_ = PointMass.makeCircle(0.2, 'bob2');
        this.mouse_ = PointMass.makeCircle(0.2, 'mouse');
        this.dragSpring_ = new Spring('dragSpring', this.mouse_, Vector.ORIGIN, this.anchor_, Vector.ORIGIN, 0, this.springStiffness_);
        const var_names = [
            MoveableDoublePendulumSim.en.ANGLE_1,
            MoveableDoublePendulumSim.en.ANGULAR_VELOCITY_1,
            MoveableDoublePendulumSim.en.ANGLE_2,
            MoveableDoublePendulumSim.en.ANGULAR_VELOCITY_2,
            VarsList.en.TIME,
            MoveableDoublePendulumSim.en.ANCHOR_X,
            MoveableDoublePendulumSim.en.ANCHOR_X_VELOCITY,
            MoveableDoublePendulumSim.en.ANCHOR_Y,
            MoveableDoublePendulumSim.en.ANCHOR_Y_VELOCITY,
            EnergyInfo.en.KINETIC_ENERGY,
            EnergyInfo.en.POTENTIAL_ENERGY,
            EnergyInfo.en.TOTAL_ENERGY
        ];
        const i18n_names = [
            MoveableDoublePendulumSim.i18n.ANGLE_1,
            MoveableDoublePendulumSim.i18n.ANGULAR_VELOCITY_1,
            MoveableDoublePendulumSim.i18n.ANGLE_2,
            MoveableDoublePendulumSim.i18n.ANGULAR_VELOCITY_2,
            VarsList.i18n.TIME,
            MoveableDoublePendulumSim.i18n.ANCHOR_X,
            MoveableDoublePendulumSim.i18n.ANCHOR_X_VELOCITY,
            MoveableDoublePendulumSim.i18n.ANCHOR_Y,
            MoveableDoublePendulumSim.i18n.ANCHOR_Y_VELOCITY,
            EnergyInfo.i18n.KINETIC_ENERGY,
            EnergyInfo.i18n.POTENTIAL_ENERGY,
            EnergyInfo.i18n.TOTAL_ENERGY
        ];
        this.setVarsList(new VarsList(var_names, i18n_names, this.getName() + '_VARS'));
        this.getVarsList().setComputed(KE, PE, TE);
        this.getSimList().add(this.anchor_, this.bob1_, this.bob2_, this.rod1_, this.rod2_);
        this.getVarsList().setValue(W1, Math.PI * 0.95);
        this.anchor_.setPosition(Vector.ORIGIN);
        this.saveInitialState();
        this.setAnchorYVelocity();
        this.modifyObjects();
        let pn;
        this.addParameter(new ParameterNumber(this, MoveableDoublePendulumSim.en.LENGTH_1, MoveableDoublePendulumSim.i18n.LENGTH_1, () => this.getLength1(), a => this.setLength1(a)));
        this.addParameter(new ParameterNumber(this, MoveableDoublePendulumSim.en.LENGTH_2, MoveableDoublePendulumSim.i18n.LENGTH_2, () => this.getLength2(), a => this.setLength2(a)));
        this.addParameter(new ParameterNumber(this, MoveableDoublePendulumSim.en.DAMPING, MoveableDoublePendulumSim.i18n.DAMPING, () => this.getDamping(), a => this.setDamping(a)));
        this.addParameter(new ParameterNumber(this, MoveableDoublePendulumSim.en.MASS_1, MoveableDoublePendulumSim.i18n.MASS_1, () => this.getMass1(), a => this.setMass1(a)));
        this.addParameter(new ParameterNumber(this, MoveableDoublePendulumSim.en.MASS_2, MoveableDoublePendulumSim.i18n.MASS_2, () => this.getMass2(), a => this.setMass2(a)));
        this.addParameter(new ParameterNumber(this, MoveableDoublePendulumSim.en.GRAVITY, MoveableDoublePendulumSim.i18n.GRAVITY, () => this.getGravity(), a => this.setGravity(a)));
        this.addParameter(new ParameterNumber(this, MoveableDoublePendulumSim.en.DRIVE_AMPLITUDE, MoveableDoublePendulumSim.i18n.DRIVE_AMPLITUDE, () => this.getDriveAmplitude(), a => this.setDriveAmplitude(a)));
        this.addParameter(new ParameterNumber(this, MoveableDoublePendulumSim.en.DRIVE_FREQUENCY, MoveableDoublePendulumSim.i18n.DRIVE_FREQUENCY, () => this.getDriveFrequency(), a => this.setDriveFrequency(a)));
        this.addParameter(new ParameterNumber(this, MoveableDoublePendulumSim.en.ANCHOR_DAMPING, MoveableDoublePendulumSim.i18n.ANCHOR_DAMPING, () => this.getAnchorDamping(), a => this.setAnchorDamping(a)));
        this.addParameter(new ParameterNumber(this, MoveableDoublePendulumSim.en.SPRING_STIFFNESS, MoveableDoublePendulumSim.i18n.SPRING_STIFFNESS, () => this.getSpringStiffness(), a => this.setSpringStiffness(a)));
        this.addParameter(pn = new ParameterNumber(this, EnergyInfo.en.PE_OFFSET, EnergyInfo.i18n.PE_OFFSET, () => this.getPEOffset(), a => this.setPEOffset(a)));
        pn.setLowerLimit(Number.NEGATIVE_INFINITY);
        pn.setSignifDigits(5);
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', length1_: ' + Util.NF(this.length1_)
            + ', length2_: ' + Util.NF(this.length2_)
            + ', gravity_: ' + Util.NF(this.gravity_)
            + ', damping_: ' + Util.NF(this.damping_)
            + ', frequency_: ' + Util.NF(this.frequency_)
            + ', amplitude_: ' + Util.NF(this.amplitude_)
            + ', anchorDamping_: ' + Util.NF(this.anchorDamping_)
            + ', springStiffness_: ' + Util.NF(this.springStiffness_)
            + ', running_: ' + this.running_
            + ', anchor_: ' + this.anchor_
            + ', bob1_: ' + this.bob1_
            + ', bob2_: ' + this.bob2_
            + ', potentialOffset_: ' + Util.NF(this.potentialOffset_)
            + super.toString();
    }
    ;
    getClassName() {
        return 'MoveableDoublePendulumSim';
    }
    ;
    setRunning(value) {
        this.running_ = value;
    }
    ;
    setAnchorYVelocity() {
        const value = Math.abs(this.frequency_) < 1E-10 ? 0 :
            -this.amplitude_ / this.frequency_;
        if (this.initialState_) {
            this.initialState_[TOPYP] = value * Math.cos(this.frequency_ * this.initialState_[TIME]);
        }
        const va = this.getVarsList();
        va.setValue(TOPYP, value * Math.cos(this.frequency_ * this.getTime()));
    }
    ;
    getEnergyInfo() {
        const L1 = this.length1_;
        const L2 = this.length2_;
        const ke = this.bob1_.getKineticEnergy() + this.bob2_.getKineticEnergy();
        const anchorY = this.anchor_.getPosition().getY();
        const y1 = this.bob1_.getPosition().getY() - anchorY;
        const y2 = this.bob2_.getPosition().getY() - anchorY;
        const pe = this.gravity_ * this.bob1_.getMass() * (y1 - -L1)
            + this.gravity_ * this.bob2_.getMass() * (y2 - (-L1 - L2));
        return new EnergyInfo(pe + this.potentialOffset_, ke);
    }
    ;
    getPEOffset() {
        return this.potentialOffset_;
    }
    setPEOffset(value) {
        this.potentialOffset_ = value;
        this.getVarsList().incrSequence(PE, TE);
        this.broadcastParameter(EnergyInfo.en.PE_OFFSET);
    }
    ;
    modifyObjects() {
        const va = this.getVarsList();
        const vars = va.getValues();
        const angle1 = Util.limitAngle(vars[W1]);
        if (angle1 != vars[W1]) {
            this.getVarsList().setValue(W1, angle1, false);
            vars[W1] = angle1;
        }
        const angle2 = Util.limitAngle(vars[W2]);
        if (angle2 != vars[W2]) {
            this.getVarsList().setValue(W2, angle2, false);
            vars[W2] = angle2;
        }
        this.moveObjects(vars);
        const ei = this.getEnergyInfo();
        va.setValue(KE, ei.getTranslational(), true);
        va.setValue(PE, ei.getPotential(), true);
        va.setValue(TE, ei.getTotalEnergy(), true);
    }
    ;
    moveObjects(vars) {
        const angle1 = vars[W1];
        const sinAngle1 = Math.sin(angle1);
        const cosAngle1 = Math.cos(angle1);
        const angle2 = vars[W2];
        const sinAngle2 = Math.sin(angle2);
        const cosAngle2 = Math.cos(angle2);
        this.anchor_.setPosition(new Vector(vars[TOPX], vars[TOPY]));
        const L1 = this.length1_;
        const L2 = this.length2_;
        const p1 = new Vector(vars[TOPX] + L1 * sinAngle1, vars[TOPY] - L1 * cosAngle1);
        this.bob1_.setPosition(p1);
        const p2 = new Vector(p1.getX() + L2 * sinAngle2, p1.getY() - L2 * cosAngle2);
        this.bob2_.setPosition(p2);
        const v1x = vars[W1P] * L1 * cosAngle1;
        const v1y = vars[W1P] * L1 * sinAngle1;
        const v2x = v1x + vars[W2P] * L2 * cosAngle2;
        const v2y = v1y + vars[W2P] * L2 * sinAngle2;
        this.bob1_.setVelocity(new Vector(v1x, v1y));
        this.bob2_.setVelocity(new Vector(v2x, v2y));
        this.rod1_.setStartPoint(this.anchor_.getPosition());
        this.rod1_.setEndPoint(p1);
        this.rod2_.setStartPoint(p1);
        this.rod2_.setEndPoint(p2);
    }
    ;
    getDragSpring() {
        return this.dragSpring_;
    }
    ;
    startDrag(simObject, location, offset, _dragBody, _modifiers) {
        if (simObject == this.anchor_) {
            this.springDragging_ = this.running_;
            if (this.springDragging_) {
                this.getSimList().add(this.dragSpring_);
            }
            this.mouseDrag(simObject, location, offset);
            return true;
        }
        else if (simObject == this.bob1_ || simObject == this.bob2_) {
            this.pendulumDragging_ = true;
            return true;
        }
        else {
            return false;
        }
    }
    ;
    mouseDrag(simObject, location, offset) {
        const va = this.getVarsList();
        const vars = va.getValues();
        const p = location.subtract(offset);
        if (simObject == this.anchor_) {
            if (this.springDragging_) {
                this.mouse_.setPosition(location);
            }
            else {
                va.setValue(TOPX, p.getX());
                va.setValue(TOPY, p.getY());
            }
        }
        else if (simObject == this.bob1_) {
            const th = Math.PI / 2 + Math.atan2(p.getY() - vars[TOPY], p.getX() - vars[TOPX]);
            va.setValue(W1, th);
            va.setValue(W1P, 0);
            va.setValue(W2P, 0);
        }
        else if (simObject == this.bob2_) {
            const p1 = this.bob1_.getPosition();
            const th = Math.PI / 2 + Math.atan2(p.getY() - p1.getY(), p.getX() - p1.getX());
            va.setValue(W2, th);
            va.setValue(W1P, 0);
            va.setValue(W2P, 0);
        }
        this.moveObjects(va.getValues());
    }
    ;
    finishDrag(_simObject, _location, _offset) {
        this.pendulumDragging_ = false;
        if (this.springDragging_) {
            this.springDragging_ = false;
            this.getSimList().remove(this.dragSpring_);
        }
    }
    ;
    handleKeyEvent(_evt, _pressed, _modifiers) {
    }
    ;
    evaluate(vars, change, _timeStep) {
        Util.zeroArray(change);
        this.moveObjects(vars);
        change[TIME] = 1;
        change[TOPX] = vars[TOPXP];
        change[TOPY] = vars[TOPYP];
        change[TOPXP] = -this.anchorDamping_ * vars[TOPXP];
        const mouse = this.mouse_.getPosition();
        if (this.springDragging_) {
            change[TOPXP] += this.springStiffness_ * (mouse.getX() - vars[TOPX]);
        }
        change[TOPYP] = -this.anchorDamping_ * vars[TOPYP] +
            this.amplitude_ * Math.sin(this.frequency_ * vars[TIME]);
        if (this.springDragging_) {
            change[TOPYP] += this.springStiffness_ * (mouse.getY() - vars[TOPY]);
        }
        if (!this.pendulumDragging_) {
            const ddx0 = change[TOPXP];
            const ddy0 = change[TOPYP];
            const th1 = vars[W1];
            const dth1 = vars[W1P];
            const th2 = vars[W2];
            const dth2 = vars[W2P];
            const m2 = this.bob2_.getMass();
            const m1 = this.bob1_.getMass();
            const L1 = this.length1_;
            const L2 = this.length2_;
            const g = this.gravity_;
            const b = this.damping_;
            const b2 = this.damping_;
            change[W1] = dth1;
            change[W1P] = -((2 * b * dth1 +
                ddx0 * L1 * (2 * m1 + m2) * Math.cos(th1) -
                ddx0 * L1 * m2 * Math.cos(th1 - 2 * th2) +
                2 * ddy0 * L1 * m1 * Math.sin(th1) + 2 * g * L1 * m1 * Math.sin(th1) +
                ddy0 * L1 * m2 * Math.sin(th1) + g * L1 * m2 * Math.sin(th1) +
                ddy0 * L1 * m2 * Math.sin(th1 - 2 * th2) +
                g * L1 * m2 * Math.sin(th1 - 2 * th2) +
                2 * dth2 * dth2 * L1 * L2 * m2 * Math.sin(th1 - th2) +
                dth1 * dth1 * L1 * L1 * m2 * Math.sin(2 * (th1 - th2))) /
                (L1 * L1 * (2 * m1 + m2 - m2 * Math.cos(2 * (th1 - th2)))));
            change[W2] = dth2;
            change[W2P] = -((2 * b * dth1 * L2 * m2 * Math.cos(th1 - th2) -
                b2 * (dth1 - dth2) * L1 * m2 * Math.cos(2 * (th1 - th2)) +
                L1 * (2 * b2 * dth1 * m1 - 2 * b2 * dth2 * m1 + b2 * dth1 * m2 -
                    b2 * dth2 * m2 +
                    ddx0 * L2 * m2 * (m1 + m2) * Math.cos(2 * th1 - th2) -
                    ddx0 * L2 * m2 * (m1 + m2) * Math.cos(th2) +
                    2 * dth1 * dth1 * L1 * L2 * m1 * m2 * Math.sin(th1 - th2) +
                    2 * dth1 * dth1 * L1 * L2 * m2 * m2 * Math.sin(th1 - th2) +
                    dth2 * dth2 * L2 * L2 * m2 * m2 * Math.sin(2 * (th1 - th2)) +
                    ddy0 * L2 * m1 * m2 * Math.sin(2 * th1 - th2) +
                    g * L2 * m1 * m2 * Math.sin(2 * th1 - th2) +
                    ddy0 * L2 * m2 * m2 * Math.sin(2 * th1 - th2) +
                    g * L2 * m2 * m2 * Math.sin(2 * th1 - th2) -
                    ddy0 * L2 * m1 * m2 * Math.sin(th2) -
                    g * L2 * m1 * m2 * Math.sin(th2) -
                    ddy0 * L2 * m2 * m2 * Math.sin(th2) - g * L2 * m2 * m2 * Math.sin(th2))) /
                (L1 * L2 * L2 * m2 * (-2 * m1 - m2 + m2 * Math.cos(2 * (th1 - th2)))));
        }
        return null;
    }
    ;
    getMass1() {
        return this.bob1_.getMass();
    }
    ;
    setMass1(value) {
        this.bob1_.setMass(value);
        this.getVarsList().incrSequence(KE, PE, TE);
        this.broadcastParameter(MoveableDoublePendulumSim.en.MASS_1);
    }
    ;
    getMass2() {
        return this.bob2_.getMass();
    }
    ;
    setMass2(value) {
        this.bob2_.setMass(value);
        this.getVarsList().incrSequence(KE, PE, TE);
        this.broadcastParameter(MoveableDoublePendulumSim.en.MASS_2);
    }
    ;
    getGravity() {
        return this.gravity_;
    }
    ;
    setGravity(value) {
        this.gravity_ = value;
        this.getVarsList().incrSequence(PE, TE);
        this.broadcastParameter(MoveableDoublePendulumSim.en.GRAVITY);
    }
    ;
    getDriveFrequency() {
        return this.frequency_;
    }
    ;
    setDriveFrequency(value) {
        this.frequency_ = value;
        this.setAnchorYVelocity();
        this.broadcastParameter(MoveableDoublePendulumSim.en.DRIVE_FREQUENCY);
    }
    ;
    getDriveAmplitude() {
        return this.amplitude_;
    }
    ;
    setDriveAmplitude(value) {
        this.amplitude_ = value;
        this.setAnchorYVelocity();
        this.broadcastParameter(MoveableDoublePendulumSim.en.DRIVE_AMPLITUDE);
    }
    ;
    getLength1() {
        return this.length1_;
    }
    ;
    setLength1(value) {
        this.length1_ = value;
        this.getVarsList().incrSequence(KE, PE, TE);
        this.broadcastParameter(MoveableDoublePendulumSim.en.LENGTH_1);
    }
    ;
    getLength2() {
        return this.length2_;
    }
    ;
    setLength2(value) {
        this.length2_ = value;
        this.getVarsList().incrSequence(KE, PE, TE);
        this.broadcastParameter(MoveableDoublePendulumSim.en.LENGTH_2);
    }
    ;
    getDamping() {
        return this.damping_;
    }
    ;
    setDamping(value) {
        this.damping_ = value;
        this.broadcastParameter(MoveableDoublePendulumSim.en.DAMPING);
    }
    ;
    getSpringStiffness() {
        return this.springStiffness_;
    }
    ;
    setSpringStiffness(value) {
        this.springStiffness_ = value;
        this.dragSpring_.setStiffness(value);
        this.getVarsList().incrSequence(PE, TE);
        this.broadcastParameter(MoveableDoublePendulumSim.en.SPRING_STIFFNESS);
    }
    ;
    getAnchorDamping() {
        return this.anchorDamping_;
    }
    ;
    setAnchorDamping(value) {
        this.anchorDamping_ = value;
        this.broadcastParameter(MoveableDoublePendulumSim.en.ANCHOR_DAMPING);
    }
    ;
}
MoveableDoublePendulumSim.en = {
    DRIVE_AMPLITUDE: 'drive amplitude',
    ANGLE_1: 'angle 1',
    ANGULAR_VELOCITY_1: 'angle velocity 1',
    ANGLE_2: 'angle 2',
    ANGULAR_VELOCITY_2: 'angle velocity 2',
    DAMPING: 'damping',
    DRIVE_FREQUENCY: 'drive frequency',
    GRAVITY: 'gravity',
    LENGTH_1: 'length 1',
    LENGTH_2: 'length 2',
    MASS_1: 'mass 1',
    MASS_2: 'mass 2',
    SPRING_STIFFNESS: 'spring stiffness',
    ANCHOR_DAMPING: 'anchor damping',
    ANCHOR_X: 'anchor X',
    ANCHOR_X_VELOCITY: 'anchor X velocity',
    ANCHOR_Y: 'anchor Y',
    ANCHOR_Y_VELOCITY: 'anchor Y velocity'
};
MoveableDoublePendulumSim.de_strings = {
    DRIVE_AMPLITUDE: 'Antriebsamplitude',
    ANGLE_1: 'Winkel 1',
    ANGULAR_VELOCITY_1: 'Winkel Geschwindigkeit 1',
    ANGLE_2: 'Winkel 2',
    ANGULAR_VELOCITY_2: 'Winkel Geschwindigkeit 2',
    DAMPING: 'Dämpfung',
    DRIVE_FREQUENCY: 'Antriebsfrequenz',
    GRAVITY: 'Gravitation',
    LENGTH_1: 'Länge 1',
    LENGTH_2: 'Länge 2',
    MASS_1: 'Masse 1',
    MASS_2: 'Masse 2',
    SPRING_STIFFNESS: 'Federsteifheit',
    ANCHOR_DAMPING: 'Anker Dämpfung',
    ANCHOR_X: 'Anker X',
    ANCHOR_X_VELOCITY: 'Anker X Geschwindigkeit',
    ANCHOR_Y: 'Anker Y',
    ANCHOR_Y_VELOCITY: 'Anker Y Geschwindigkeit'
};
MoveableDoublePendulumSim.i18n = Util.LOCALE === 'de' ? MoveableDoublePendulumSim.de_strings : MoveableDoublePendulumSim.en;
Util.defineGlobal('sims$pendulum$MoveableDoublePendulumSim', MoveableDoublePendulumSim);
