import { AbstractApp } from '../common/AbstractApp.js';
import { CheckBoxControl } from '../../lab/controls/CheckBoxControl.js';
import { DisplayArc } from '../../lab/view/DisplayArc.js';
import { DisplayLine } from '../../lab/view/DisplayLine.js';
import { DisplayShape } from '../../lab/view/DisplayShape.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { PendulumSim } from './PendulumSim.js';
import { SimpleAdvance } from '../../lab/model/SimpleAdvance.js';
import { SliderControl } from '../../lab/controls/SliderControl.js';
import { Util } from '../../lab/util/Util.js';
export class PendulumApp extends AbstractApp {
    constructor(elem_ids) {
        Util.setErrorHandler();
        const simRect = new DoubleRect(-2, -2.2, 2, 1.5);
        const sim = new PendulumSim();
        const advance = new SimpleAdvance(sim);
        super(elem_ids, simRect, sim, advance, sim, sim);
        this.rod = new DisplayLine(this.simList.getConcreteLine('rod'));
        this.displayList.add(this.rod);
        this.drive = new DisplayArc(this.simList.getArc('drive'));
        this.displayList.add(this.drive);
        this.bob = new DisplayShape(this.simList.getPointMass('bob'));
        this.bob.setFillStyle('blue');
        this.displayList.add(this.bob);
        sim.getVarsList().getVariable(PendulumSim.en.ANGLE).setValue(Math.PI / 8);
        sim.modifyObjects();
        this.addPlaybackControls();
        let pn = sim.getParameterNumber(PendulumSim.en.LENGTH);
        this.addControl(new SliderControl(pn, 0.1, 10.1, true));
        pn = sim.getParameterNumber(PendulumSim.en.DAMPING);
        this.addControl(new SliderControl(pn, 0, 1, false));
        pn = sim.getParameterNumber(PendulumSim.en.MASS);
        this.addControl(new SliderControl(pn, 0.2, 20.2, true));
        pn = sim.getParameterNumber(PendulumSim.en.DRIVE_AMPLITUDE);
        this.addControl(new SliderControl(pn, 0, 10, false));
        pn = sim.getParameterNumber(PendulumSim.en.DRIVE_FREQUENCY);
        this.addControl(new SliderControl(pn, 0, 10, false));
        pn = sim.getParameterNumber(PendulumSim.en.GRAVITY);
        this.addControl(new SliderControl(pn, 0, 20, false));
        const pb = sim.getParameterBoolean(PendulumSim.en.LIMIT_ANGLE);
        this.addControl(new CheckBoxControl(pb));
        this.addStandardControls();
        this.makeEasyScript();
        this.addURLScriptButton();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', bob: ' + this.bob.toStringShort()
            + ', rod: ' + this.rod.toStringShort()
            + ', drive: ' + this.drive.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'PendulumApp';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('rod|drive|bob', myName + '.');
        this.terminal.addRegex('PendulumSim', 'sims$pendulum$', false);
    }
    ;
}
Util.defineGlobal('sims$pendulum$PendulumApp', PendulumApp);
