import { AbstractSubject } from '../../lab/util/AbstractSubject.js';
import { CheckBoxControl } from '../../lab/controls/CheckBoxControl.js';
import { Clock } from '../../lab/util/Clock.js';
import { CommonControls } from '../common/CommonControls.js';
import { CompareGraph } from '../common/CompareGraph.js';
import { CompareTimeGraph } from '../common/CompareTimeGraph.js';
import { ConcreteLine } from '../../lab/model/ConcreteLine.js';
import { DisplayClock } from '../../lab/view/DisplayClock.js';
import { DisplayLine } from '../../lab/view/DisplayLine.js';
import { DisplayShape } from '../../lab/view/DisplayShape.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { EnergyBarGraph } from '../../lab/graph/EnergyBarGraph.js';
import { GenericObserver } from '../../lab/util/Observe.js';
import { GraphLine } from '../../lab/graph/GraphLine.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { ParameterNumber } from '../../lab/util/Observe.js';
import { PendulumSim } from './PendulumSim.js';
import { Polygon } from '../../lab/engine2D/Polygon.js';
import { ReactionPendulumSim } from './ReactionPendulumSim.js';
import { SimController } from '../../lab/app/SimController.js';
import { SimList } from '../../lab/model/SimList.js';
import { SimpleAdvance } from '../../lab/model/SimpleAdvance.js';
import { SimRunner } from '../../lab/app/SimRunner.js';
import { SimView } from '../../lab/view/SimView.js';
import { SliderControl } from '../../lab/controls/SliderControl.js';
import { TabLayout } from '../common/TabLayout.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
export class ReactionPendulumApp extends AbstractSubject {
    ;
    constructor(elem_ids) {
        Util.setErrorHandler();
        super('APP');
        this.separation = 0.5;
        this.startAngle = 5 * Math.PI / 7;
        this.pendulumLength = 1.5;
        this.radius = 0.4;
        this.simRect = new DoubleRect(-2, -2, 2, 2.7);
        this.layout = new TabLayout(elem_ids);
        this.layout.getSimCanvas().setBackground('black');
        this.terminal = this.layout.getTerminal();
        const simCanvas = this.layout.getSimCanvas();
        this.simView = new SimView('simView', this.simRect);
        this.displayList = this.simView.getDisplayList();
        simCanvas.addView(this.simView);
        this.statusView = new SimView('status', new DoubleRect(-10, -10, 10, 10));
        simCanvas.addView(this.statusView);
        this.sim1 = new PendulumSim('SIM_1');
        const va1 = this.sim1.getVarsList();
        this.sim1.setLength(this.classicLength());
        this.sim1.setDriveAmplitude(0);
        this.sim1.setPivot(new Vector(this.separation, 0));
        this.sim1.setDamping(0);
        this.sim1.setGravity(3);
        this.sim1.setMass(0.1);
        va1.setValue(0, this.startAngle);
        va1.setValue(1, 0);
        this.sim1.saveInitialState();
        this.simList1 = this.sim1.getSimList();
        new GenericObserver(this.sim1, (_evt) => this.sim1.modifyObjects(), 'modifyObjects after parameter or variable change');
        const displayBob1 = new DisplayShape(this.simList1.getPointMass('bob'));
        displayBob1.setFillStyle('#3cf');
        displayBob1.setDragable(false);
        this.displayList.add(displayBob1);
        const displayRod1 = new DisplayLine(this.simList1.getConcreteLine('rod'));
        displayRod1.setColor('#39c');
        displayRod1.setThickness(3);
        this.displayList.add(displayRod1);
        this.simList2 = new SimList();
        this.simList2.addObserver(this);
        this.sim2 = new ReactionPendulumSim(this.pendulumLength, this.radius, this.startAngle, 'SIM_2', this.simList2);
        this.sim2.setMass(this.sim1.getMass());
        this.sim2.setGravity(this.sim1.getGravity());
        this.sim2.setDamping(0);
        this.terminal.setAfterEval(() => {
            this.sim1.modifyObjects();
            this.sim2.modifyObjects();
        });
        new GenericObserver(this.sim2, (_evt) => this.sim2.modifyObjects(), 'modifyObjects after parameter or variable change');
        this.simCtrl = new SimController(simCanvas, null);
        this.axes = CommonControls.makeAxes(this.simView, true);
        this.advance1 = new SimpleAdvance(this.sim1);
        this.advance2 = new SimpleAdvance(this.sim2);
        this.simRun = new SimRunner(this.advance1);
        this.simRun.addStrategy(this.advance2);
        this.simRun.addCanvas(simCanvas);
        this.clock = this.simRun.getClock();
        let pb;
        let pn;
        this.addControl(CommonControls.makePlaybackControls(this.simRun));
        this.addParameter(pn = new ParameterNumber(this, ReactionPendulumSim.en.START_ANGLE, ReactionPendulumSim.i18n.START_ANGLE, () => this.getStartAngle(), a => this.setStartAngle(a)));
        this.addControl(new NumericControl(pn));
        this.addParameter(pn = new ParameterNumber(this, ReactionPendulumSim.en.LENGTH, ReactionPendulumSim.i18n.LENGTH, () => this.getLength(), a => this.setLength(a)));
        this.addControl(new NumericControl(pn));
        this.addParameter(pn = new ParameterNumber(this, ReactionPendulumSim.en.RADIUS, ReactionPendulumSim.i18n.RADIUS, () => this.getRadius(), a => this.setRadius(a)));
        this.addControl(new NumericControl(pn));
        this.addParameter(pn = new ParameterNumber(this, ReactionPendulumApp.en.SEPARATION, ReactionPendulumApp.i18n.SEPARATION, () => this.getSeparation(), a => this.setSeparation(a)));
        this.addControl(new SliderControl(pn, 0, 2, false));
        pn = this.sim1.getParameterNumber(PendulumSim.en.MASS);
        this.addControl(new SliderControl(pn, 0.001, 10, true));
        pn = this.sim1.getParameterNumber(PendulumSim.en.GRAVITY);
        this.addControl(new SliderControl(pn, 0, 20, false));
        this.energyGraph1 = new EnergyBarGraph(this.sim1);
        this.energyGraph1.potentialColor = '#039';
        this.energyGraph1.translationColor = '#06c';
        this.energyGraph1.rotationColor = '#6cf';
        this.showEnergyParam1 = CommonControls.makeShowEnergyParam(this.energyGraph1, this.statusView, this);
        this.addControl(new CheckBoxControl(this.showEnergyParam1));
        this.energyGraph2 = new EnergyBarGraph(this.sim2);
        this.energyGraph2.potentialColor = '#903';
        this.energyGraph2.translationColor = '#f33';
        this.energyGraph2.rotationColor = '#f99';
        this.energyGraph2.setPosition(new Vector(0, 6));
        this.showEnergyParam2 = CommonControls.makeShowEnergyParam(this.energyGraph2, this.statusView, this, ReactionPendulumApp.en.SHOW_ENERGY_2, ReactionPendulumApp.i18n.SHOW_ENERGY_2);
        this.addControl(new CheckBoxControl(this.showEnergyParam2));
        this.displayClock = new DisplayClock(() => this.sim1.getTime(), () => this.clock.getRealTime(), 2, 2);
        this.displayClock.setPosition(new Vector(8, 4));
        pb = CommonControls.makeShowClockParam(this.displayClock, this.statusView, this);
        this.addControl(new CheckBoxControl(pb));
        const panzoom_simview = CommonControls.makePanZoomControls(this.simView, true, () => this.simView.setSimRect(this.simRect));
        this.layout.getSimDiv().appendChild(panzoom_simview);
        pb = CommonControls.makeShowPanZoomParam(panzoom_simview, this);
        pb.setValue(false);
        this.addControl(new CheckBoxControl(pb));
        pn = this.simRun.getParameterNumber(SimRunner.en.TIME_STEP);
        this.addControl(new NumericControl(pn));
        pn = this.simRun.getClock().getParameterNumber(Clock.en.TIME_RATE);
        this.addControl(new NumericControl(pn));
        const bm = CommonControls.makeBackgroundMenu(this.layout.getSimCanvas());
        this.addControl(bm);
        const translate = (v1) => {
            switch (v1) {
                case 0: return 4;
                case 1: return 5;
                case 2: return 6;
                case 3: return -1;
                case 4: return 7;
                case 5: return 8;
                case 6: return 9;
                default: throw '';
            }
        };
        const line1 = new GraphLine('GRAPH_LINE_1', va1);
        line1.setXVariable(0);
        line1.setYVariable(1);
        line1.setColor('blue');
        line1.setDrawingMode("lines");
        const va2 = this.sim2.getVarsList();
        const line2 = new GraphLine('GRAPH_LINE_2', va2);
        line2.setXVariable(translate(0));
        line2.setYVariable(translate(1));
        line2.setColor('red');
        line2.setDrawingMode("lines");
        const paramY1 = line1.getParameterNumber(GraphLine.en.Y_VARIABLE);
        const paramX1 = line1.getParameterNumber(GraphLine.en.X_VARIABLE);
        new GenericObserver(line1, (evt) => {
            if (evt == paramY1) {
                line2.setYVariable(translate(paramY1.getValue()));
            }
            else if (evt == paramX1) {
                line2.setXVariable(translate(paramX1.getValue()));
            }
        }, 'keep line1\'s X and Y variable in sync with line2');
        this.graph = new CompareGraph(line1, line2, this.layout.getGraphCanvas(), this.layout.getGraphControls(), this.layout.getGraphDiv(), this.simRun);
        const timeLine1 = new GraphLine('TIME_LINE_1', va1);
        timeLine1.setYVariable(0);
        timeLine1.setColor('blue');
        timeLine1.setDrawingMode("lines");
        const timeLine2 = new GraphLine('TIME_LINE_2', va2);
        timeLine2.setYVariable(translate(0));
        timeLine2.setColor('red');
        timeLine2.setDrawingMode("lines");
        const timeParamY1 = timeLine1.getParameterNumber(GraphLine.en.Y_VARIABLE);
        new GenericObserver(timeLine1, (evt) => {
            if (evt == timeParamY1) {
                timeLine2.setYVariable(translate(timeParamY1.getValue()));
            }
        }, 'keep timeLine2\'s Y variable in sync with timeLine1');
        this.timeGraph = new CompareTimeGraph(timeLine1, timeLine2, this.layout.getTimeGraphCanvas(), this.layout.getTimeGraphControls(), this.layout.getTimeGraphDiv(), this.simRun);
        new GenericObserver(this.sim1, (evt) => {
            if (evt instanceof ParameterNumber) {
                if (evt.nameEquals(PendulumSim.en.GRAVITY)) {
                    this.sim2.setGravity(evt.getValue());
                }
                else if (evt.nameEquals(PendulumSim.en.MASS)) {
                    this.sim2.setMass(evt.getValue());
                }
            }
            else if (evt.nameEquals('RESET')) {
                this.sim2.reset();
            }
        }, 'synchronize sim2 parameters to match parameters of sim1');
        let subjects = [
            this,
            this.sim1,
            this.sim2,
            this.simRun,
            this.clock,
            this.simView,
            this.statusView,
            this.sim1.getVarsList(),
            this.sim2.getVarsList()
        ];
        subjects = subjects.concat(this.layout.getSubjects(), this.graph.getSubjects(), this.timeGraph.getSubjects());
        this.easyScript = CommonControls.makeEasyScript(subjects, [], this.simRun, this.terminal);
        this.addControl(CommonControls.makeURLScriptButton(this.easyScript, this.simRun));
        this.graph.addControl(CommonControls.makeURLScriptButton(this.easyScript, this.simRun));
        this.timeGraph.addControl(CommonControls.makeURLScriptButton(this.easyScript, this.simRun));
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', sim1: ' + this.sim1.toStringShort()
            + ', sim2: ' + this.sim2.toStringShort()
            + ', terminal: ' + this.terminal
            + ', graph: ' + this.graph
            + ', timeGraph: ' + this.timeGraph
            + super.toString();
    }
    ;
    getClassName() {
        return 'ReactionPendulumApp';
    }
    ;
    addControl(control) {
        this.layout.addControl(control);
    }
    ;
    observe(event) {
        if (event.getSubject() == this.simList2) {
            const obj = event.getValue();
            if (event.nameEquals(SimList.OBJECT_ADDED)) {
                if (this.displayList.find(obj)) {
                    return;
                }
                if (obj instanceof Polygon) {
                    const d = new DisplayShape(obj);
                    d.setDrawCenterOfMass(true);
                    d.setFillStyle('#f66');
                    d.setZIndex(-1);
                    this.displayList.add(d);
                }
                else if (obj instanceof ConcreteLine) {
                    const dl = new DisplayLine(obj);
                    dl.setThickness(4);
                    if (obj.nameEquals('rod')) {
                        dl.setColor('#f99');
                    }
                    else {
                        dl.setColor('green');
                        dl.setLineDash([3, 5]);
                    }
                    this.displayList.add(dl);
                }
            }
            else if (event.nameEquals(SimList.OBJECT_REMOVED)) {
                const d = this.displayList.find(obj);
                if (d) {
                    this.displayList.remove(d);
                }
            }
        }
    }
    ;
    reset() {
        this.sim2.config(this.pendulumLength, this.radius, this.startAngle);
        this.sim1.setLength(this.classicLength());
        this.sim1.getVarsList().setValue(0, this.startAngle);
        this.sim1.getVarsList().setValue(1, 0);
        this.sim1.getVarsList().setValue(2, 0);
        this.sim1.saveInitialState();
        this.simRun.reset();
    }
    ;
    getStartAngle() {
        return this.startAngle;
    }
    ;
    setStartAngle(value) {
        if (this.startAngle != value) {
            this.startAngle = value;
            this.reset();
            this.broadcastParameter(ReactionPendulumSim.en.START_ANGLE);
        }
    }
    ;
    classicLength() {
        return this.pendulumLength + this.radius * this.radius / (2 * this.pendulumLength);
    }
    ;
    getLength() {
        return this.pendulumLength;
    }
    ;
    setLength(value) {
        if (this.pendulumLength != value) {
            this.pendulumLength = value;
            this.reset();
            this.broadcastParameter(ReactionPendulumSim.en.LENGTH);
        }
    }
    ;
    getRadius() {
        return this.radius;
    }
    ;
    setRadius(value) {
        if (this.radius != value) {
            this.radius = value;
            this.reset();
            this.broadcastParameter(ReactionPendulumSim.en.RADIUS);
        }
    }
    ;
    getSeparation() {
        return this.separation;
    }
    ;
    setSeparation(value) {
        if (this.separation != value) {
            this.separation = value;
            this.sim1.setPivot(new Vector(value, 0));
            this.broadcastParameter(ReactionPendulumApp.en.SEPARATION);
        }
    }
    ;
    defineNames(myName) {
        this.terminal.addRegex('advance2|advance1|axes|clock|displayList'
            + '|displayClock|energyGraph|graph|layout|easyScript|sim2|sim1'
            + '|simCtrl|simList2|simList1|simRun|simView|statusView|terminal|timeGraph', myName + '.');
        this.terminal.addRegex('simCanvas', myName + '.layout.');
    }
    ;
    setup() {
        this.clock.resume();
        this.terminal.parseURL();
        this.sim1.saveInitialState();
        this.sim1.modifyObjects();
    }
    ;
    start() {
        this.simRun.startFiring();
    }
    ;
    eval(script, output = true) {
        return this.terminal.eval(script, output);
    }
    ;
}
ReactionPendulumApp.en = {
    SEPARATION: 'separation',
    SHOW_ENERGY_2: 'show energy 2'
};
ReactionPendulumApp.de_strings = {
    SEPARATION: 'Abstand',
    SHOW_ENERGY_2: 'Energieanzeige 2'
};
ReactionPendulumApp.i18n = Util.LOCALE === 'de' ? ReactionPendulumApp.de_strings : ReactionPendulumApp.en;
Util.defineGlobal('sims$pendulum$ReactionPendulumApp', ReactionPendulumApp);
