import { AbstractApp } from '../common/AbstractApp.js';
import { DisplayConnector } from '../../lab/view/DisplayConnector.js';
import { DisplayShape } from '../../lab/view/DisplayShape.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { EnergyInfo } from '../../lab/model/EnergySystem.js';
import { GenericObserver } from '../../lab/util/Observe.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { RigidDoublePendulumSim } from './RigidDoublePendulumSim.js';
import { SimpleAdvance } from '../../lab/model/SimpleAdvance.js';
import { SliderControl } from '../../lab/controls/SliderControl.js';
import { Util } from '../../lab/util/Util.js';
export class RigidDoublePendulumApp extends AbstractApp {
    constructor(elem_ids, centered) {
        Util.setErrorHandler();
        const parts = centered ? RigidDoublePendulumSim.makeCentered(0.25 * Math.PI, 0)
            : RigidDoublePendulumSim.makeOffset(0.25 * Math.PI, 0);
        const simRect = new DoubleRect(-2, -2, 2, 2);
        const sim = new RigidDoublePendulumSim(parts);
        const advance = new SimpleAdvance(sim);
        super(elem_ids, simRect, sim, advance, null, sim);
        new GenericObserver(sim, (evt) => {
            if (evt.nameEquals('RESET')) {
                this.clock.setTime(sim.getTime());
            }
        }, 'sync clock time on reset');
        let ds = new DisplayShape();
        ds.setFillStyle('');
        ds.setStrokeStyle('blue');
        ds.setDrawCenterOfMass(true);
        ds.setThickness(3);
        this.protoBob = ds;
        this.bob0 = new DisplayShape(parts.bodies[0], this.protoBob);
        this.displayList.add(this.bob0);
        this.bob1 = new DisplayShape(parts.bodies[1], this.protoBob);
        this.displayList.add(this.bob1);
        this.joint0 = new DisplayConnector(parts.joints[0]);
        this.displayList.add(this.joint0);
        this.joint1 = new DisplayConnector(parts.joints[1]);
        this.displayList.add(this.joint1);
        this.addPlaybackControls();
        let pn = sim.getParameterNumber(RigidDoublePendulumSim.en.GRAVITY);
        this.addControl(new SliderControl(pn, 0, 20, false));
        pn = sim.getParameterNumber(RigidDoublePendulumSim.en.ANGLE_1);
        this.addControl(new SliderControl(pn, -Math.PI, Math.PI, false));
        pn = sim.getParameterNumber(RigidDoublePendulumSim.en.ANGLE_2);
        this.addControl(new SliderControl(pn, -Math.PI, Math.PI, false));
        pn = sim.getParameterNumber(EnergyInfo.en.PE_OFFSET);
        this.addControl(new NumericControl(pn));
        this.addStandardControls();
        this.makeEasyScript();
        this.addURLScriptButton();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', bob0: ' + this.bob0.toStringShort()
            + ', bob1: ' + this.bob1.toStringShort()
            + ', joint0: ' + this.joint0.toStringShort()
            + ', joint1: ' + this.joint1.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'RigidDoublePendulumApp';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('joint0|joint1|bob1|bob0|protoBob', myName + '.');
    }
    ;
}
Util.defineGlobal('sims$pendulum$RigidDoublePendulumApp', RigidDoublePendulumApp);
