import { AbstractApp } from '../common/AbstractApp.js';
import { DisplayLine } from '../../lab/view/DisplayLine.js';
import { DisplayShape } from '../../lab/view/DisplayShape.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { GenericObserver } from '../../lab/util/Observe.js';
import { PendulumSim } from './PendulumSim.js';
import { SimpleAdvance } from '../../lab/model/SimpleAdvance.js';
import { SliderControl } from '../../lab/controls/SliderControl.js';
import { TabLayout3 } from '../common/TabLayout3.js';
import { Util } from '../../lab/util/Util.js';
import { VectorGraph } from '../../lab/graph/VectorGraph.js';
export class VectorGraphPendulumApp extends AbstractApp {
    constructor(elem_ids) {
        Util.setErrorHandler();
        const simRect = new DoubleRect(-2, -2.2, 2, 1.5);
        const sim = new PendulumSim();
        const advance = new SimpleAdvance(sim);
        super(elem_ids, simRect, sim, advance, sim, sim);
        sim.setDriveAmplitude(0);
        sim.setDamping(0);
        sim.setGravity(9.8);
        sim.setLength(1);
        this.rod = new DisplayLine(this.simList.getConcreteLine('rod'));
        this.displayList.add(this.rod);
        this.bob = new DisplayShape(this.simList.getPointMass('bob'));
        this.bob.setFillStyle('blue');
        this.displayList.add(this.bob);
        this.graph.line.setDrawingMode("lines");
        this.vectorGraph = new VectorGraph(sim, 0, 1);
        this.vectorGraph.setScreenRect(this.graph.view.getScreenRect());
        this.graph.displayList.add(this.vectorGraph);
        new GenericObserver(this.graph.line, (_evt) => {
            const yVar = this.graph.line.getYVariable();
            const xVar = this.graph.line.getXVariable();
            const isOK = yVar == 1 && xVar == 0;
            const isVis = this.graph.displayList.contains(this.vectorGraph);
            if (isOK && !isVis) {
                this.graph.displayList.add(this.vectorGraph);
            }
            else if (!isOK && isVis) {
                this.graph.displayList.remove(this.vectorGraph);
            }
        }, 'remove VectorGraph when other variables shown');
        this.addPlaybackControls();
        let pn = sim.getParameterNumber(PendulumSim.en.LENGTH);
        this.addControl(new SliderControl(pn, 0.1, 10.1, true));
        pn = sim.getParameterNumber(PendulumSim.en.DAMPING);
        this.addControl(new SliderControl(pn, 0, 10, false));
        pn = sim.getParameterNumber(PendulumSim.en.MASS);
        this.addControl(new SliderControl(pn, 0.2, 20.2, true));
        pn = sim.getParameterNumber(PendulumSim.en.GRAVITY);
        this.addControl(new SliderControl(pn, 0, 20, false));
        this.addStandardControls();
        this.makeEasyScript();
        this.addURLScriptButton();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', bob: ' + this.bob.toStringShort()
            + ', rod: ' + this.rod.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'VectorGraphPendulumApp';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('rod|bob', myName + '.');
    }
    ;
    makeLayout(elem_ids) {
        const canvasWidth = 800;
        const canvasHeight = Math.round(canvasWidth * this.simRect.getHeight() /
            this.simRect.getWidth());
        return new TabLayout3(elem_ids, canvasWidth, canvasHeight);
    }
    ;
}
Util.defineGlobal('sims$pendulum$VectorGraphPendulumApp', VectorGraphPendulumApp);
