import { AbstractSubject } from '../../lab/util/AbstractSubject.js';
import { NumericalPath } from '../../lab/model/NumericalPath.js';
import { ParameterString } from '../../lab/util/Observe.js';
import { Util } from '../../lab/util/Util.js';
export class PathSelector extends AbstractSubject {
    constructor(hasPath, paths) {
        super('PATH_SELECTOR');
        this.hasPath_ = hasPath;
        this.paths_ = paths;
        const path = this.hasPath_.getPath();
        this.pathName_ = path != null ? path.getName() : '';
        const names = paths.map(p => p.getName());
        const localNames = paths.map(p => p.getName(true));
        const ps = new ParameterString(this, PathSelector.en.PATH, PathSelector.i18n.PATH, () => this.getPathName(), a => this.setPathName(a), localNames, names);
        this.addParameter(ps);
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', pathName: ' + this.pathName_
            + ', paths: ['
            + this.paths_.map(p => p.getName())
            + ']' + super.toString();
    }
    ;
    toStringShort() {
        return super.toStringShort().slice(0, -1)
            + ', hasPath: ' + this.hasPath_.toStringShort() + '}';
    }
    ;
    getClassName() {
        return 'PathSelector';
    }
    ;
    getPathName() {
        return this.pathName_;
    }
    ;
    setPathName(value) {
        value = Util.toName(value);
        if (value != this.pathName_) {
            for (let i = 0, len = this.paths_.length; i < len; i++) {
                const path = this.paths_[i];
                if (path.nameEquals(value)) {
                    this.hasPath_.setPath(new NumericalPath(path));
                    this.pathName_ = path.getName();
                    this.broadcastParameter(PathSelector.en.PATH);
                    return;
                }
            }
            throw 'unknown path ' + value;
        }
        ;
    }
    ;
    update() {
        for (let i = 0, len = this.paths_.length; i < len; i++) {
            const path = this.paths_[i];
            if (path.nameEquals(this.pathName_)) {
                this.hasPath_.setPath(new NumericalPath(path));
                this.broadcastParameter(PathSelector.en.PATH);
                return;
            }
        }
        throw 'unknown path ' + this.pathName_;
    }
    ;
}
PathSelector.en = {
    PATH: 'path'
};
PathSelector.de_strings = {
    PATH: 'Pfad'
};
PathSelector.i18n = Util.LOCALE === 'de' ? PathSelector.de_strings : PathSelector.en;
Util.defineGlobal('sims$roller$PathSelector', PathSelector);
