import { PathPoint } from '../../lab/model/PathPoint.js';
import { Util } from '../../lab/util/Util.js';
export class RollerCollision {
    constructor(ball, path, time) {
        this.pathPoint_ = new PathPoint();
        this.distance_ = NaN;
        this.mustHandle_ = false;
        this.accuracy_ = 0.005;
        this.impulse = NaN;
        this.velocity = NaN;
        this.ball_ = ball;
        this.path_ = path;
        this.detectedTime_ = time;
        this.updateCollision(time);
    }
    ;
    toString() {
        return 'RollerCollision{'
            + 'distance=' + Util.NF(this.distance_)
            + ', accuracy=' + Util.NF(this.accuracy_)
            + ', detectedTime=' + Util.NF(this.detectedTime_)
            + ', impulse: ' + Util.NF5(this.impulse)
            + ', velocity: ' + Util.NF5(this.velocity)
            + ', position=' + this.ball_.getPosition()
            + '}';
    }
    ;
    getPathPoint() {
        return this.pathPoint_;
    }
    ;
    closeEnough(_allowTiny) {
        return Math.abs(this.distance_) <= this.accuracy_;
    }
    ;
    isTouching() {
        return true;
    }
    ;
    isColliding() {
        return this.distance_ < -this.accuracy_;
    }
    ;
    getDistance() {
        return this.distance_;
    }
    ;
    getDetectedTime() {
        return this.detectedTime_;
    }
    ;
    getEstimatedTime() {
        return NaN;
    }
    ;
    bilateral() {
        return false;
    }
    ;
    contact() {
        return false;
    }
    ;
    illegalState() {
        return this.distance_ < -this.accuracy_;
    }
    ;
    setNeedsHandling(needsHandling) {
        this.mustHandle_ = needsHandling;
    }
    ;
    needsHandling() {
        return this.mustHandle_;
    }
    ;
    updateCollision(_time) {
        this.pathPoint_ = new PathPoint();
        const pos = this.ball_.getPosition();
        this.pathPoint_.x = pos.getX();
        this.path_.map_x_to_y_p(this.pathPoint_);
        this.path_.map_p_to_slope(this.pathPoint_);
        this.distance_ = pos.getY() - this.pathPoint_.getY();
        const normal = this.pathPoint_.getNormal();
        this.velocity = this.ball_.getVelocity().dotProduct(normal);
    }
    ;
    getImpulse() {
        return this.impulse;
    }
    ;
    getVelocity() {
        return this.velocity;
    }
    ;
}
Util.defineGlobal('sims$roller$RollerCollision', RollerCollision);
