import { AbstractApp } from '../common/AbstractApp.js';
import { CollisionAdvance } from '../../lab/model/CollisionAdvance.js';
import { CommonControls } from '../common/CommonControls.js';
import { DisplayPath } from '../../lab/view/DisplayPath.js';
import { DisplayShape } from '../../lab/view/DisplayShape.js';
import { DisplaySpring } from '../../lab/view/DisplaySpring.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { GenericObserver } from '../../lab/util/Observe.js';
import { HumpPath } from './HumpPath.js';
import { SimView } from '../../lab/view/SimView.js';
import { NumericalPath } from '../../lab/model/NumericalPath.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { RollerFlightSim } from './RollerFlightSim.js';
import { Util } from '../../lab/util/Util.js';
export class RollerFlightApp extends AbstractApp {
    constructor(elem_ids) {
        Util.setErrorHandler();
        const path = new NumericalPath(new HumpPath());
        const simRect = new DoubleRect(-6, -6, 6, 6);
        const sim = new RollerFlightSim(path);
        const advance = new CollisionAdvance(sim);
        super(elem_ids, simRect, sim, advance, sim, sim);
        this.path = path;
        this.layout.getSimCanvas().setBackground('black');
        this.layout.getSimCanvas().setAlpha(CommonControls.SHORT_TRAILS);
        this.ball1 = new DisplayShape(this.simList.getPointMass('ball1'));
        this.ball1.setFillStyle('blue');
        this.displayList.add(this.ball1);
        this.anchor = new DisplayShape(this.simList.getPointMass('anchor'));
        this.anchor.setFillStyle('red');
        this.displayList.add(this.anchor);
        this.spring = new DisplaySpring(this.simList.getSpring('spring'));
        this.spring.setWidth(0.2);
        this.spring.setColorCompressed('red');
        this.spring.setColorExpanded('#6f6');
        this.displayList.add(this.spring);
        this.displayPath = new DisplayPath();
        this.displayPath.setScreenRect(this.simView.getScreenRect());
        this.displayPath.addPath(this.path);
        this.displayList.add(this.displayPath);
        this.simView.setSimRect(this.path.getBoundsWorld().scale(1.1));
        const trackVar = sim.getVarsList().getVariable(6);
        new GenericObserver(sim.getVarsList(), (evt) => {
            if (evt == trackVar) {
                this.ball1.setFillStyle(trackVar.getValue() > 0 ? 'blue' : 'red');
            }
        }, 'change color of ball when in free flight');
        new GenericObserver(this.simView, (evt) => {
            if (evt.nameEquals(SimView.SCREEN_RECT_CHANGED)) {
                this.displayPath.setScreenRect(this.simView.getScreenRect());
            }
        }, 'resize displayPath when screen rect changes');
        this.addPlaybackControls();
        let pn = sim.getParameterNumber(RollerFlightSim.en.ELASTICITY);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(RollerFlightSim.en.STICKINESS);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(RollerFlightSim.en.GRAVITY);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(RollerFlightSim.en.MASS);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(RollerFlightSim.en.DAMPING);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(RollerFlightSim.en.SPRING_STIFFNESS);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(RollerFlightSim.en.SPRING_LENGTH);
        this.addControl(new NumericControl(pn));
        this.addStandardControls();
        this.graph.line.setXVariable(0);
        this.graph.line.setYVariable(1);
        this.timeGraph.line1.setYVariable(0);
        this.timeGraph.line2.setYVariable(1);
        this.makeEasyScript();
        this.addURLScriptButton();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', ball1: ' + this.ball1.toStringShort()
            + ', anchor: ' + this.anchor.toStringShort()
            + ', spring: ' + this.spring.toStringShort()
            + ', path: ' + this.path.toStringShort()
            + ', displayPath: ' + this.displayPath.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'RollerFlightApp';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('ball1|anchor|spring|path|displayPath', myName + '.');
    }
    ;
}
Util.defineGlobal('sims$roller$RollerFlightApp', RollerFlightApp);
