import { AbstractApp } from '../common/AbstractApp.js';
import { CardioidPath } from './CardioidPath.js';
import { ChoiceControl } from '../../lab/controls/ChoiceControl.js';
import { CirclePath } from './CirclePath.js';
import { DisplayShape } from '../../lab/view/DisplayShape.js';
import { DisplaySpring } from '../../lab/view/DisplaySpring.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { FlatPath } from './FlatPath.js';
import { HumpPath } from './HumpPath.js';
import { LemniscatePath } from './LemniscatePath.js';
import { LoopTheLoopPath } from './LoopTheLoopPath.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { OvalPath } from './OvalPath.js';
import { PathObserver } from './PathObserver.js';
import { PathSelector } from './PathSelector.js';
import { RollerSingleSim } from './RollerSingleSim.js';
import { SimpleAdvance } from '../../lab/model/SimpleAdvance.js';
import { SpiralPath } from './SpiralPath.js';
import { Util } from '../../lab/util/Util.js';
export class RollerSpringApp extends AbstractApp {
    constructor(elem_ids) {
        Util.setErrorHandler();
        const simRect = new DoubleRect(-6, -6, 6, 6);
        const sim = new RollerSingleSim(true);
        const advance = new SimpleAdvance(sim);
        super(elem_ids, simRect, sim, advance, sim, sim);
        this.ball1 = new DisplayShape(this.simList.getPointMass('ball1'));
        this.ball1.setFillStyle('blue');
        this.displayList.add(this.ball1);
        this.anchor = new DisplayShape(this.simList.getPointMass('anchor'));
        this.anchor.setFillStyle('red');
        this.displayList.add(this.anchor);
        let ds = new DisplaySpring(this.simList.getSpring('spring'));
        ds.setWidth(0.2);
        ds.setColorCompressed('#f00');
        ds.setColorExpanded('#6f6');
        this.spring = ds;
        this.displayList.add(this.spring);
        this.paths = [
            new HumpPath(),
            new LoopTheLoopPath(),
            new CirclePath(3.0),
            new OvalPath(),
            new LemniscatePath(2.0),
            new CardioidPath(3.0),
            new SpiralPath(),
            new FlatPath()
        ];
        this.pathSelect = new PathSelector(sim, this.paths);
        this.pathObserver = new PathObserver(this.simList, this.simView, a => this.setSimRect(a));
        this.pathSelect.setPathName(HumpPath.en.NAME);
        this.addPlaybackControls();
        let ps = this.pathSelect.getParameterString(PathSelector.en.PATH);
        this.addControl(new ChoiceControl(ps));
        let pn = sim.getParameterNumber(RollerSingleSim.en.GRAVITY);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(RollerSingleSim.en.DAMPING);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(RollerSingleSim.en.MASS);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(RollerSingleSim.en.SPRING_STIFFNESS);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(RollerSingleSim.en.SPRING_LENGTH);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(RollerSingleSim.en.SPRING_DAMPING);
        this.addControl(new NumericControl(pn));
        this.addStandardControls();
        this.graph.line.setXVariable(0);
        this.graph.line.setYVariable(1);
        this.timeGraph.line1.setYVariable(0);
        this.timeGraph.line2.setYVariable(1);
        this.makeEasyScript([this.simView]);
        this.addURLScriptButton();
        this.pathSelect.addObserver(this);
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', anchor: ' + this.anchor.toStringShort()
            + ', ball1: ' + this.ball1.toStringShort()
            + ', spring: ' + this.spring.toStringShort()
            + ', pathSelect: ' + this.pathSelect.toStringShort()
            + ', paths: [ ' + this.paths + ' ]'
            + super.toString();
    }
    ;
    getClassName() {
        return 'RollerSpringApp';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('ball1|anchor|spring|paths|pathSelect', myName + '.');
    }
    ;
    getSubjects() {
        return super.getSubjects().concat(this.pathSelect);
    }
    ;
    observe(event) {
        if (event.getSubject() == this.pathSelect) {
            this.easyScript.update();
            this.sim.modifyObjects();
        }
    }
    ;
    setSimRect(simRect) {
        this.simRect = simRect;
        this.simView.setSimRect(simRect);
    }
    ;
}
Util.defineGlobal('sims$roller$RollerSpringApp', RollerSpringApp);
