import { AbstractApp } from '../common/AbstractApp.js';
import { CollideBlocksSim } from './CollideBlocksSim.js';
import { CollisionAdvance } from '../../lab/model/CollisionAdvance.js';
import { DisplayShape } from '../../lab/view/DisplayShape.js';
import { DisplaySpring } from '../../lab/view/DisplaySpring.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { Util } from '../../lab/util/Util.js';
export class CollideBlocksApp extends AbstractApp {
    constructor(elem_ids) {
        Util.setErrorHandler();
        const simRect = new DoubleRect(-0.5, -2, 7.5, 2);
        const sim = new CollideBlocksSim();
        const advance = new CollisionAdvance(sim);
        super(elem_ids, simRect, sim, advance, sim, sim);
        this.protoBlock = new DisplayShape();
        this.protoBlock.setFillStyle('blue');
        this.protoWall = new DisplayShape();
        this.protoWall.setFillStyle('lightGray');
        this.protoSpring = new DisplaySpring();
        this.block1 = new DisplayShape(this.simList.getPointMass('block1'), this.protoBlock);
        this.displayList.add(this.block1);
        this.block2 = new DisplayShape(this.simList.getPointMass('block2'), this.protoBlock);
        this.displayList.add(this.block2);
        this.wallLeft = new DisplayShape(this.simList.getPointMass('wallLeft'), this.protoWall);
        this.displayList.add(this.wallLeft);
        this.wallRight = new DisplayShape(this.simList.getPointMass('wallRight'), this.protoWall);
        this.displayList.add(this.wallRight);
        this.spring1 = new DisplaySpring(this.simList.getSpring('spring1'), this.protoSpring);
        this.displayList.add(this.spring1);
        this.spring2 = new DisplaySpring(this.simList.getSpring('spring2'), this.protoSpring);
        this.displayList.add(this.spring2);
        this.addPlaybackControls();
        let pn = sim.getParameterNumber(CollideBlocksSim.en.MASS_1);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(CollideBlocksSim.en.MASS_2);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(CollideBlocksSim.en.DAMPING);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(CollideBlocksSim.en.STIFFNESS_1);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(CollideBlocksSim.en.LENGTH_1);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(CollideBlocksSim.en.STIFFNESS_2);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(CollideBlocksSim.en.LENGTH_2);
        this.addControl(new NumericControl(pn));
        this.addStandardControls();
        this.graph.line.setXVariable(0);
        this.graph.line.setYVariable(2);
        this.timeGraph.line1.setYVariable(0);
        this.timeGraph.line2.setYVariable(2);
        this.makeEasyScript();
        this.addURLScriptButton();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', block1: ' + this.block1.toStringShort()
            + ', block2: ' + this.block2.toStringShort()
            + ', wallLeft: ' + this.wallLeft.toStringShort()
            + ', wallRight: ' + this.wallRight.toStringShort()
            + ', spring1: ' + this.spring1.toStringShort()
            + ', spring2: ' + this.spring2.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'CollideBlocksApp';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('block1|block2|wallLeft|wallRight|spring1|spring2'
            + '|protoBlock|protoWall|protoSpring', myName + '.');
    }
    ;
}
Util.defineGlobal('sims$springs$CollideBlocksApp', CollideBlocksApp);
