import { AbstractApp } from '../common/AbstractApp.js';
import { DangleStickSim } from './DangleStickSim.js';
import { DisplayLine } from '../../lab/view/DisplayLine.js';
import { DisplayShape } from '../../lab/view/DisplayShape.js';
import { DisplaySpring } from '../../lab/view/DisplaySpring.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { SimpleAdvance } from '../../lab/model/SimpleAdvance.js';
import { SliderControl } from '../../lab/controls/SliderControl.js';
import { Util } from '../../lab/util/Util.js';
export class DangleStickApp extends AbstractApp {
    constructor(elem_ids) {
        Util.setErrorHandler();
        const simRect = new DoubleRect(-4, -4, 4, 2);
        const sim = new DangleStickSim();
        const advance = new SimpleAdvance(sim);
        super(elem_ids, simRect, sim, advance, sim, null);
        this.protoMass = new DisplayShape();
        this.protoMass.setFillStyle('blue');
        this.stick = new DisplayLine(this.simList.getConcreteLine('stick'));
        this.displayList.add(this.stick);
        this.bob1 = new DisplayShape(this.simList.getPointMass('bob1'), this.protoMass);
        this.displayList.add(this.bob1);
        this.bob2 = new DisplayShape(this.simList.getPointMass('bob2'), this.protoMass);
        this.displayList.add(this.bob2);
        this.spring = new DisplaySpring(this.simList.getSpring('spring'));
        this.spring.setWidth(0.3);
        this.displayList.add(this.spring);
        this.addPlaybackControls();
        let pn = sim.getParameterNumber(DangleStickSim.en.GRAVITY);
        this.addControl(new SliderControl(pn, 0, 20, false));
        pn = sim.getParameterNumber(DangleStickSim.en.MASS1);
        this.addControl(new SliderControl(pn, 0.2, 20.2, true));
        pn = sim.getParameterNumber(DangleStickSim.en.MASS2);
        this.addControl(new SliderControl(pn, 0.2, 20.2, true));
        pn = sim.getParameterNumber(DangleStickSim.en.STICK_LENGTH);
        this.addControl(new SliderControl(pn, 0.1, 10.1, true));
        pn = sim.getParameterNumber(DangleStickSim.en.SPRING_REST_LENGTH);
        this.addControl(new SliderControl(pn, 0.1, 10.1, true));
        pn = sim.getParameterNumber(DangleStickSim.en.SPRING_STIFFNESS);
        this.addControl(new SliderControl(pn, 0.1, 100.1, true));
        this.addStandardControls();
        this.makeEasyScript();
        this.addURLScriptButton();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', bob1: ' + this.bob1.toStringShort()
            + ', bob2: ' + this.bob2.toStringShort()
            + ', stick: ' + this.stick.toStringShort()
            + ', spring: ' + this.spring.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'DangleStickApp';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('stick|bob1|bob2|spring|protoMass', myName + '.');
    }
    ;
}
Util.defineGlobal('sims$springs$DangleStickApp', DangleStickApp);
