import { AbstractApp } from '../common/AbstractApp.js';
import { CollisionAdvance } from '../../lab/model/CollisionAdvance.js';
import { DisplayShape } from '../../lab/view/DisplayShape.js';
import { DisplaySpring } from '../../lab/view/DisplaySpring.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { MoleculeSim, KE, PE, TE } from './MoleculeSim.js';
import { ParameterNumber } from '../../lab/util/Observe.js';
import { PointMass } from '../../lab/model/PointMass.js';
import { SliderControl } from '../../lab/controls/SliderControl.js';
import { Spring } from '../../lab/model/Spring.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
export class Molecule1App extends AbstractApp {
    constructor(elem_ids) {
        Util.setErrorHandler();
        const simRect = new DoubleRect(-6, -6, 6, 6);
        const sim = new MoleculeSim();
        const advance = new CollisionAdvance(sim);
        super(elem_ids, simRect, sim, advance, sim, sim);
        this.layout.getSimCanvas().setBackground('white');
        this.atom1_ = PointMass.makeCircle(0.5, 'atom1');
        this.atom1_.setMass(0.5);
        this.atom1_.setVelocityX(1.5);
        this.atom2_ = PointMass.makeCircle(0.5, 'atom2');
        this.atom2_.setMass(0.5);
        this.atom2_.setPositionY(1.7);
        this.spring_ = new Spring('spring', this.atom1_, Vector.ORIGIN, this.atom2_, Vector.ORIGIN, 2.0, 6.0);
        this.spring_.setDamping(0);
        sim.addAtom(this.atom1_);
        sim.addAtom(this.atom2_);
        sim.addSpring(this.spring_);
        sim.saveInitialState();
        this.walls = new DisplayShape(this.simList.getPointMass('walls'));
        this.walls.setFillStyle('');
        this.walls.setStrokeStyle('gray');
        this.displayList.add(this.walls);
        this.spring = new DisplaySpring(this.spring_);
        this.spring.setWidth(0.3);
        this.spring.setThickness(3);
        this.displayList.add(this.spring);
        this.atom1 = new DisplayShape(this.atom1_);
        this.atom1.setFillStyle('blue');
        this.displayList.add(this.atom1);
        this.atom2 = new DisplayShape(this.atom2_);
        this.atom2.setFillStyle('red');
        this.displayList.add(this.atom2);
        this.addPlaybackControls();
        let pn = sim.getParameterNumber(MoleculeSim.en.GRAVITY);
        this.addControl(new SliderControl(pn, 0, 20, false));
        pn = sim.getParameterNumber(MoleculeSim.en.DAMPING);
        this.addControl(new SliderControl(pn, 0, 1, false));
        pn = sim.getParameterNumber(MoleculeSim.en.ELASTICITY);
        this.addControl(new SliderControl(pn, 0, 1, false));
        this.addParameter(pn = new ParameterNumber(this, Molecule1App.en.MASS1, Molecule1App.i18n.MASS1, () => this.getMass1(), a => this.setMass1(a)));
        this.addControl(new SliderControl(pn, 0.2, 20.2, true));
        this.addParameter(pn = new ParameterNumber(this, Molecule1App.en.MASS2, Molecule1App.i18n.MASS2, () => this.getMass2(), a => this.setMass2(a)));
        this.addControl(new SliderControl(pn, 0.2, 20.2, true));
        this.addParameter(pn = new ParameterNumber(this, Molecule1App.en.SPRING_LENGTH, Molecule1App.i18n.SPRING_LENGTH, () => this.getLength(), a => this.setLength(a)));
        this.addControl(new SliderControl(pn, 0.1, 10.1, true));
        this.addParameter(pn = new ParameterNumber(this, Molecule1App.en.SPRING_STIFFNESS, Molecule1App.i18n.SPRING_STIFFNESS, () => this.getStiffness(), a => this.setStiffness(a)));
        this.addControl(new SliderControl(pn, 0, 100, false));
        this.addStandardControls();
        this.makeEasyScript();
        this.addURLScriptButton();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', atom1: ' + this.atom1.toStringShort()
            + ', atom2: ' + this.atom2.toStringShort()
            + ', spring: ' + this.spring.toStringShort()
            + ', walls: ' + this.walls.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'Molecule1App';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('walls|atom1|atom2|spring', myName + '.');
    }
    ;
    getMass1() {
        return this.atom1_.getMass();
    }
    ;
    setMass1(value) {
        this.atom1_.setMass(value);
        this.sim.getVarsList().incrSequence(KE, PE, TE);
        this.broadcastParameter(Molecule1App.en.MASS1);
    }
    ;
    getMass2() {
        return this.atom2_.getMass();
    }
    ;
    setMass2(value) {
        this.atom2_.setMass(value);
        this.sim.getVarsList().incrSequence(KE, PE, TE);
        this.broadcastParameter(Molecule1App.en.MASS2);
    }
    ;
    getLength() {
        return this.spring_.getRestLength();
    }
    ;
    setLength(value) {
        this.spring_.setRestLength(value);
        this.sim.getVarsList().incrSequence(PE, TE);
        this.broadcastParameter(Molecule1App.en.SPRING_LENGTH);
    }
    ;
    getStiffness() {
        return this.spring_.getStiffness();
    }
    ;
    setStiffness(value) {
        this.setStiffness(value);
        this.sim.getVarsList().incrSequence(PE, TE);
        this.broadcastParameter(Molecule1App.en.SPRING_STIFFNESS);
    }
    ;
}
Molecule1App.en = {
    MASS1: 'blue mass',
    MASS2: 'red mass',
    SPRING_LENGTH: 'spring length',
    SPRING_STIFFNESS: 'spring stiffness'
};
Molecule1App.de_strings = {
    MASS1: 'blaue Masse',
    MASS2: 'rote Masse',
    SPRING_LENGTH: 'Federlänge',
    SPRING_STIFFNESS: 'Federsteifheit'
};
Molecule1App.i18n = Util.LOCALE === 'de' ? Molecule1App.de_strings : Molecule1App.en;
Util.defineGlobal('sims$springs$Molecule1App', Molecule1App);
