import { Util } from '../../lab/util/Util.js';
;
export class MoleculeCollision {
    constructor(atom, wall, side, time) {
        this.distance_ = NaN;
        this.mustHandle_ = false;
        this.targetGap_ = 0.005;
        this.accuracy_ = 0.005;
        this.impulse = NaN;
        this.atom = atom;
        this.wall = wall;
        this.side = side;
        this.detectedTime_ = time;
        this.updateCollision(time);
    }
    ;
    toString() {
        return 'MoleculeCollision{'
            + 'distance: ' + Util.NF5(this.distance_)
            + ', targetGap: ' + Util.NF5(this.targetGap_)
            + ', accuracy: ' + Util.NF5(this.accuracy_)
            + ', detectedTime: ' + Util.NF7(this.detectedTime_)
            + ', impulse: ' + Util.NF5(this.impulse)
            + ', mustHandle: ' + this.mustHandle_
            + ', atom: ' + this.atom
            + ', wall: ' + this.wall
            + ', side: ' + this.side
            + '}';
    }
    ;
    closeEnough(allowTiny) {
        if (allowTiny) {
            return this.distance_ > 0 && this.distance_ < this.targetGap_ + this.accuracy_;
        }
        else {
            return Math.abs(this.distance_ - this.targetGap_) <= this.accuracy_;
        }
    }
    ;
    bilateral() {
        return false;
    }
    ;
    contact() {
        return false;
    }
    ;
    getDetectedTime() {
        return this.detectedTime_;
    }
    ;
    getDistance() {
        return this.distance_;
    }
    ;
    getEstimatedTime() {
        return NaN;
    }
    ;
    getImpulse() {
        return this.impulse;
    }
    ;
    getVelocity() {
        const v = this.atom.getVelocity();
        switch (this.side) {
            case "top":
                return -v.getY();
            case "bottom":
                return v.getY();
            case "left":
                return v.getX();
            case "right":
                return -v.getX();
            default:
                throw 'no such side ' + this.side;
        }
    }
    ;
    illegalState() {
        return this.distance_ < 0;
    }
    ;
    isColliding() {
        return this.distance_ < this.targetGap_ - this.accuracy_;
    }
    ;
    isTouching() {
        return this.distance_ < 2 * this.targetGap_;
    }
    ;
    needsHandling() {
        return this.mustHandle_;
    }
    ;
    setNeedsHandling(needsHandling) {
        this.mustHandle_ = needsHandling;
    }
    ;
    updateCollision(_time) {
        const a = this.atom.getBoundsWorld();
        const w = this.wall.getBoundsWorld();
        switch (this.side) {
            case "top":
                this.distance_ = w.getTop() - a.getTop();
                break;
            case "bottom":
                this.distance_ = a.getBottom() - w.getBottom();
                break;
            case "left":
                this.distance_ = a.getLeft() - w.getLeft();
                break;
            case "right":
                this.distance_ = w.getRight() - a.getRight();
                break;
            default:
                throw 'no such side ' + this.side;
        }
    }
    ;
}
Util.defineGlobal('sims$springs$MoleculeCollision', MoleculeCollision);
