import { CollisionAdvance } from '../lab/model/CollisionAdvance.js';
import { ContactSim } from '../lab/engine2D/ContactSim.js';
import { DampingLaw } from '../lab/model/DampingLaw.js';
import { Gravity2Law } from '../lab/model/Gravity2Law.js';
import { GravityLaw } from '../lab/model/GravityLaw.js';
import { ImpulseSim } from '../lab/engine2D/ImpulseSim.js';
import { RungeKutta } from '../lab/model/RungeKutta.js';
import { Shapes } from '../lab/engine2D/Shapes.js';
import { makeBlockRoundEdge, makeConcaveCirclePoly } from './TestShapes.js';
import { Vector } from '../lab/util/Vector.js';
import { Walls } from '../lab/engine2D/Walls.js';
import { schedule, setTestName } from './TestRig.js';
import { makeVars, setBodyVars, runTest, runExceptionTest } from './Engine2DTestRig.js';
const groupName = 'CircleStraightTest.';
export class CircleStraightTest {
    constructor() { throw ''; }
    ;
    static test() {
        schedule(CircleStraightTest.ball_block_collide);
        schedule(CircleStraightTest.ball_block_attract);
        schedule(CircleStraightTest.ball_block_contact);
        schedule(CircleStraightTest.circle_arc_block);
        schedule(CircleStraightTest.concave_ball_block_collide);
        schedule(CircleStraightTest.concave_ball_block_contact);
        schedule(CircleStraightTest.concave_ball_block_contact_2);
        schedule(CircleStraightTest.rotating_block_vs_ball_0);
        schedule(CircleStraightTest.rotating_block_vs_ball_1);
        schedule(CircleStraightTest.ball_falls_on_floor_stuck);
        schedule(CircleStraightTest.wedged_ball);
        schedule(CircleStraightTest.elastic_balls);
    }
    ;
    static commonSetup1(sim, advance, damping) {
        damping = damping === undefined ? 0 : damping;
        sim.addForceLaw(new DampingLaw(damping, 0.5, sim.getSimList()));
        sim.setCollisionAccuracy(0.6);
        sim.setCollisionHandling("serial grouped lastpass");
        sim.setRandomSeed(99999);
        sim.setDistanceTol(0.01);
        sim.setVelocityTol(0.5);
        if (sim instanceof ContactSim) {
            sim.setExtraAccel("velocity");
        }
        advance.setTimeStep(0.025);
        advance.setDiffEqSolver(new RungeKutta(sim));
    }
    ;
    static ball_block_collide_setup(sim, advance) {
        CircleStraightTest.commonSetup1(sim, advance);
        const body0 = Shapes.makeBall(0.75, 'ball');
        body0.setCenterOfMass(new Vector(0, 0.2));
        body0.setPosition(new Vector(-2, 2), 0);
        body0.setVelocity(new Vector(1, -1), 1);
        sim.addBody(body0);
        const body1 = Shapes.makeBlock(1, 1, 'block');
        body1.setPosition(new Vector(0, 0), 0);
        sim.addBody(body1);
        sim.setElasticity(1.0);
    }
    ;
    static ball_block_collide() {
        setTestName(groupName + 'ball_block_collide');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        CircleStraightTest.ball_block_collide_setup(sim, advance);
        const vars = makeVars(6 * 2);
        setBodyVars(sim, vars, 0, -1.135972, 0.0179439, 1.1812653, 0.0028806, 3.1087039, 1.0499788);
        setBodyVars(sim, vars, 1, 2.135972, 0.9820561, -2.1812653, -1.0028806, 0.1358799, 0.0624735);
        runTest(sim, advance, 3.0, vars, 0.00001, 0.0, 0.00001, -1);
    }
    ;
    static ball_block_attract_setup(sim, advance) {
        CircleStraightTest.commonSetup1(sim, advance);
        const body0 = Shapes.makeBall(0.75, 'ball');
        body0.setCenterOfMass(new Vector(0, 0.2));
        body0.setPosition(new Vector(-2, 2), 0);
        body0.setVelocity(new Vector(0.1, -0.1), 1);
        sim.addBody(body0);
        const body1 = Shapes.makeBlock(1, 1, 'block');
        body1.setPosition(new Vector(0, 0), 0);
        body1.setVelocity(new Vector(-0.1, 0.1), -1);
        sim.addBody(body1);
        sim.setElasticity(0.8);
        sim.addForceLaw(new Gravity2Law(3.0, sim.getSimList()));
    }
    ;
    static ball_block_attract() {
        setTestName(groupName + 'ball_block_attract');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        CircleStraightTest.ball_block_attract_setup(sim, advance);
        const vars = makeVars(6 * 2);
        setBodyVars(sim, vars, 0, -1.3699699, 0.2138555, 1.5307076, -0.2372712, 33.3931299, 1.7136164);
        setBodyVars(sim, vars, 1, -0.6300301, -0.2138555, 0.4692924, 0.2372712, -7.2779759, -1.8957024);
        runTest(sim, advance, 20.0, vars, 0.0001);
    }
    ;
    static ball_block_contact_setup(sim, advance) {
        CircleStraightTest.commonSetup1(sim, advance);
        const body0 = Shapes.makeBall(0.75, 'ball');
        body0.setCenterOfMass(new Vector(0, 0.2));
        body0.setPosition(new Vector(-0.2525, 0), 0);
        body0.setVelocity(new Vector(0, 0.6), 1);
        sim.addBody(body0);
        const body1 = Shapes.makeBlock(1, 3, 'block');
        body1.setPosition(new Vector(1, 0), 0);
        body1.setVelocity(new Vector(0, -0.6), 0);
        sim.addBody(body1);
        sim.setElasticity(0.8);
        sim.addForceLaw(new Gravity2Law(3.0, sim.getSimList()));
        const collisions = [];
        sim.findCollisions(collisions, sim.getVarsList().getValues(), 0);
        sim.handleCollisions(collisions);
    }
    ;
    static ball_block_contact() {
        setTestName(groupName + 'ball_block_contact');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        CircleStraightTest.ball_block_contact_setup(sim, advance);
        const vars = makeVars(6 * 2);
        setBodyVars(sim, vars, 0, -0.2749218, 0.0874195, 0.4044095, 0.1208178, 0.4380613, 0.0903604);
        setBodyVars(sim, vars, 1, 1.0224218, -0.0874195, -0.4044095, -0.1208178, -0.0766021, -0.3218581);
        runTest(sim, advance, 1.0, vars, 0.00001);
        setBodyVars(sim, vars, 0, -0.0405174, -0.1600652, 0.4408221, -0.0927029, -2.9119182, -1.956407);
        setBodyVars(sim, vars, 1, 0.7880174, 0.1600652, -0.4408221, 0.0927029, -0.4529687, -0.165526);
        runTest(sim, advance, 4.0, vars, 0.00001, 0.0, 0.00001, 0);
    }
    ;
    static circle_arc_block_setup(sim, advance) {
        CircleStraightTest.commonSetup1(sim, advance);
        advance.setTimeStep(0.01);
        const p = makeBlockRoundEdge();
        p.setPosition(new Vector(0, -1), 0);
        sim.addBody(p);
        const p2 = Shapes.makeBlock(1, 1, 'block');
        p2.setPosition(new Vector(2.5, 0.3), Math.PI / 8 - Math.PI / 24);
        sim.addBody(p2);
        sim.setElasticity(0.8);
        sim.addForceLaw(new Gravity2Law(3.0, sim.getSimList()));
    }
    ;
    static circle_arc_block() {
        setTestName(groupName + 'circle_arc_block');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        CircleStraightTest.circle_arc_block_setup(sim, advance);
        const vars = makeVars(6 * 2);
        setBodyVars(sim, vars, 0, 0.8343664, 0.164365, -0.7928884, -0.2694134, -1.0677983, -0.8300614);
        setBodyVars(sim, vars, 1, 1.6656336, -0.164365, 0.0928884, 0.2694134, 0.5765963, 0.188213);
        runTest(sim, advance, 8.0, vars, 0.0001);
        runTest(sim, advance, 12.0, null, NaN, 0.0, 0.0001, 0);
    }
    ;
    static concave_ball_block_collide_setup(sim, advance) {
        CircleStraightTest.commonSetup1(sim, advance);
        advance.setTimeStep(0.01);
        const p = makeConcaveCirclePoly();
        p.setPosition(new Vector(0, -2), 0);
        sim.addBody(p);
        const p2 = Shapes.makeBlock(1, 1, 'block');
        p2.setPosition(new Vector(0.5, -0.22), Math.PI / 8 - Math.PI / 24);
        sim.addBody(p2);
        sim.setElasticity(0.8);
        sim.addForceLaw(new Gravity2Law(3.0, sim.getSimList()));
    }
    ;
    static concave_ball_block_collide() {
        setTestName(groupName + 'concave_ball_block_collide');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        CircleStraightTest.concave_ball_block_collide_setup(sim, advance);
        const vars = makeVars(6 * 2);
        setBodyVars(sim, vars, 0, 0.3091927, 0.5475717, -1.5539282, 0.0193363, 0.084485, -0.2984758);
        setBodyVars(sim, vars, 1, 0.1908073, -0.5475717, -0.6660718, -0.0193363, -1.5117501, -1.2518051);
        runTest(sim, advance, 8.0, vars, 0.00001);
        runTest(sim, advance, 12.0, null, NaN, 0.0, 0.001, 0);
    }
    ;
    static concave_ball_block_contact_setup(sim, advance) {
        CircleStraightTest.commonSetup1(sim, advance);
        advance.setTimeStep(0.01);
        const p = makeConcaveCirclePoly();
        p.setPosition(new Vector(0, -2.3228757), 0);
        p.setElasticity(0.8);
        sim.addBody(p);
        const p2 = Shapes.makeBlock(1, 1, 'block');
        p2.setElasticity(0.8);
        const angle = Math.PI / 16;
        let dist = 1.5;
        let error = Infinity;
        while (Math.abs(error) > 1e-8) {
            p2.setPosition(new Vector(0, 0), angle);
            p2.saveOldCoords();
            p.saveOldCoords();
            p2.setPosition(new Vector(dist * Math.sin(angle), -dist * Math.cos(angle)), angle);
            const collisions = [];
            p.checkCollision(collisions, p2, 0);
            if (collisions.length == 0) {
                dist += 0.01;
            }
            else {
                error = Infinity;
                for (let i = 0, len = collisions.length; i < len; i++) {
                    const c = collisions[i];
                    const depth = c.distanceToHalfGap() - 0.001;
                    if (depth < error) {
                        error = depth;
                    }
                }
                dist += error;
            }
        }
        sim.addBody(p2);
        sim.addForceLaw(new Gravity2Law(10.0, sim.getSimList()));
    }
    ;
    static concave_ball_block_contact() {
        setTestName(groupName + 'concave_ball_block_contact');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        CircleStraightTest.concave_ball_block_contact_setup(sim, advance);
        const vars = makeVars(6 * 2);
        setBodyVars(sim, vars, 0, 0.0976832, -0.7068779, -2.3078333, 0.0053314, -0.0510627, 0.3847691);
        setBodyVars(sim, vars, 1, 0.1813533, 0.7068779, -1.417854, -0.0053314, -0.0243965, 1.6129892);
        runTest(sim, advance, 8.0, vars, 0.0001, 0.0, 0.0001, 0);
    }
    ;
    static concave_ball_block_contact_2() {
        setTestName(groupName + 'concave_ball_block_contact_2');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        CircleStraightTest.concave_ball_block_contact_setup(sim, advance);
        advance.setTimeStep(0.0025);
        const vars = makeVars(6 * 2);
        setBodyVars(sim, vars, 0, 0.0976759, -0.7068614, -2.3078333, 0.0053102, -0.0510571, 0.3847591);
        setBodyVars(sim, vars, 1, 0.1813607, 0.7068614, -1.417854, -0.0053102, -0.024377, 1.6129487);
        runTest(sim, advance, 8.0, vars, 0.000001, 0.0, 0.000001, 0);
    }
    ;
    static rotating_block_vs_ball_init(sim, advance) {
        CircleStraightTest.commonSetup1(sim, advance);
        const body0 = Shapes.makeBall(0.75, 'ball');
        body0.setPosition(new Vector(-0.4, 0), 0);
        body0.setVelocity(new Vector(0, 0), 0);
        sim.addBody(body0);
        const body1 = Shapes.makeBlock(1, 3, 'block');
        body1.setPosition(new Vector(1.5, 0), -Math.PI / 4);
        body1.setVelocity(new Vector(-0.2, 0), 0.5);
        sim.addBody(body1);
        sim.setElasticity(0.8);
    }
    ;
    static rotating_block_vs_ball_1_setup(sim, advance) {
        CircleStraightTest.rotating_block_vs_ball_init(sim, advance);
        sim.setElasticity(1);
    }
    ;
    static rotating_block_vs_ball_1() {
        setTestName(groupName + 'rotating_block_vs_ball_1');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        CircleStraightTest.rotating_block_vs_ball_1_setup(sim, advance);
        const vars = makeVars(6 * 2);
        setBodyVars(sim, vars, 0, -1.6324684, -0.2331973, -0.7357376, -0.1458555, -0, -0);
        setBodyVars(sim, vars, 1, 1.1324684, 0.0331973, 0.7357376, 0.1458555, 1.965865, 0.2512581);
        runTest(sim, advance, 8.0, vars, 0.00001);
    }
    ;
    static rotating_block_vs_ball_0_setup(sim, advance) {
        CircleStraightTest.rotating_block_vs_ball_init(sim, advance);
        sim.setElasticity(0);
    }
    ;
    static rotating_block_vs_ball_0() {
        setTestName(groupName + 'rotating_block_vs_ball_0');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        CircleStraightTest.rotating_block_vs_ball_0_setup(sim, advance);
        const vars = makeVars(6 * 2);
        setBodyVars(sim, vars, 0, -1.6274159, -0.2325424, -0.7371557, -0.1463887, -0, -0);
        setBodyVars(sim, vars, 1, 1.1274159, 0.0325424, 0.7371557, 0.1463887, 1.964511, 0.2505587);
        runTest(sim, advance, 8.0, vars, 0.00001);
    }
    ;
    static ball_falls_on_floor_setup(sim, advance) {
        CircleStraightTest.commonSetup1(sim, advance);
        sim.clearForceLaws();
        const body0 = Shapes.makeBall(0.5, 'ball');
        body0.setPosition(new Vector(0, 0), 0);
        sim.addBody(body0);
        const body1 = Shapes.makeBlock(10, 1, 'wall');
        body1.setMass(Infinity);
        body1.setPosition(new Vector(0, -2.5), 0);
        sim.addBody(body1);
        const floor = body1.getTopWorld();
        body0.setZeroEnergyLevel(floor + body0.getMinHeight());
        const g = new GravityLaw(3.0, sim.getSimList());
        sim.addForceLaw(g);
        sim.setElasticity(0.8);
    }
    ;
    static ball_falls_on_floor_stuck() {
        setTestName(groupName + 'ball_falls_on_floor_stuck');
        const sim = new ImpulseSim();
        const advance = new CollisionAdvance(sim);
        CircleStraightTest.ball_falls_on_floor_setup(sim, advance);
        runExceptionTest(advance, 15);
    }
    ;
    static wedged_ball_setup(sim, advance) {
        CircleStraightTest.commonSetup1(sim, advance, 0);
        sim.setExtraAccel("velocity_and_distance");
        sim.setCollisionAccuracy(0.6);
        const body0 = Shapes.makeBall(1.5, 'ball');
        body0.setCenterOfMass(new Vector(0.4, 0.2));
        body0.setPosition(new Vector(-5.5 + 1.5 + .01, -3), 0);
        sim.addBody(body0);
        const body1 = Shapes.makeBlock(1, 1, 'fixed block');
        body1.setPosition(new Vector(-2.5, -5), 0);
        body1.setMass(Infinity);
        sim.addBody(body1);
        const gravity = new GravityLaw(3.0, sim.getSimList());
        sim.addForceLaw(gravity);
        const zel = Walls.make(sim, 12.0, 12.0, 1.0);
        gravity.setZeroEnergyLevel(zel);
        sim.setElasticity(0.8);
    }
    ;
    static wedged_ball() {
        setTestName(groupName + 'wedged_ball');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        CircleStraightTest.wedged_ball_setup(sim, advance);
        const vars = makeVars(1 * 6);
        setBodyVars(sim, vars, 0, -4.9277059, -0.1631422, -4.4398357, 0.6282979, -3.3497083, -1.4515518);
        runTest(sim, advance, 3, vars, 0.00001);
        runTest(sim, advance, 8.0, null, NaN, 0.0, 0.0002, 0);
    }
    ;
    static elastic_balls_setup(sim, advance) {
        CircleStraightTest.commonSetup1(sim, advance, 0);
        sim.setExtraAccel("velocity_and_distance");
        let body = Shapes.makeBall(0.8, 'ball1');
        body.setPosition(new Vector(-5, 2));
        body.setElasticity(1.0);
        sim.addBody(body);
        body = Shapes.makeBall(0.8, 'ball2');
        body.setPosition(new Vector(-3, 2));
        body.setElasticity(0.8);
        sim.addBody(body);
        body = Shapes.makeBall(0.8, 'ball3');
        body.setPosition(new Vector(-1, 2));
        body.setElasticity(0.6);
        sim.addBody(body);
        body = Shapes.makeBall(0.8, 'ball4');
        body.setPosition(new Vector(1, 2));
        body.setElasticity(0.4);
        sim.addBody(body);
        body = Shapes.makeBall(0.8, 'ball5');
        body.setPosition(new Vector(3, 2));
        body.setElasticity(0.2);
        sim.addBody(body);
        body = Shapes.makeBall(0.8, 'ball6');
        body.setPosition(new Vector(5, 2));
        body.setElasticity(0);
        sim.addBody(body);
        const gravity = new GravityLaw(3.0, sim.getSimList());
        sim.addForceLaw(gravity);
        const zel = Walls.make(sim, 12.0, 12.0, 1.0);
        gravity.setZeroEnergyLevel(zel);
    }
    ;
    static elastic_balls() {
        setTestName(groupName + 'elastic_balls');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        CircleStraightTest.elastic_balls_setup(sim, advance);
        const vars = makeVars(6 * 6);
        setBodyVars(sim, vars, 0, -5, 0, -0.3044753, -3.7184475, 0, 0);
        setBodyVars(sim, vars, 1, -3, 0, -3.5265603, -1.1428554, 0, 0);
        setBodyVars(sim, vars, 2, -1, 0, -5.195, -0, 0, 0);
        setBodyVars(sim, vars, 3, 1, 0, -5.195, 0, 0, 0);
        setBodyVars(sim, vars, 4, 3, 0, -5.195, -0, 0, 0);
        setBodyVars(sim, vars, 5, 5, 0, -5.195, -0, 0, 0);
        runTest(sim, advance, 10, vars, 0.00001, NaN, NaN, 17);
    }
    ;
}
