import { CollisionAdvance } from '../lab/model/CollisionAdvance.js';
import { ContactSim } from '../lab/engine2D/ContactSim.js';
import { DampingLaw } from '../lab/model/DampingLaw.js';
import { GravityLaw } from '../lab/model/GravityLaw.js';
import { ImpulseSim } from '../lab/engine2D/ImpulseSim.js';
import { JointUtil } from '../lab/engine2D/JointUtil.js';
import { RungeKutta } from '../lab/model/RungeKutta.js';
import { Shapes } from '../lab/engine2D/Shapes.js';
import { Vector } from '../lab/util/Vector.js';
import { Walls } from '../lab/engine2D/Walls.js';
import { schedule, setTestName } from './TestRig.js';
import { makeVars, setBodyVars, runTest } from './Engine2DTestRig.js';
const groupName = 'MultipleCollisionTest.';
export class MultipleCollisionTest {
    constructor() { throw ''; }
    ;
    static test() {
        schedule(MultipleCollisionTest.test1_0.bind(null, "simultaneous"));
        schedule(MultipleCollisionTest.test1_0.bind(null, "hybrid"));
        schedule(MultipleCollisionTest.test1_1.bind(null, "serial grouped lastpass"));
        schedule(MultipleCollisionTest.test2_0.bind(null, "simultaneous", MultipleCollisionTest.BALL));
        schedule(MultipleCollisionTest.test2_0.bind(null, "simultaneous", MultipleCollisionTest.BLOCK));
        schedule(MultipleCollisionTest.test2_1.bind(null, "serial grouped lastpass", MultipleCollisionTest.BALL));
        schedule(MultipleCollisionTest.test2_1.bind(null, "hybrid", MultipleCollisionTest.BALL));
        schedule(MultipleCollisionTest.test2_1.bind(null, "hybrid", MultipleCollisionTest.BLOCK));
        schedule(MultipleCollisionTest.test3_0.bind(null, "simultaneous"));
        schedule(MultipleCollisionTest.test3_0.bind(null, "hybrid"));
        schedule(MultipleCollisionTest.test3_1.bind(null, "serial grouped lastpass"));
        schedule(MultipleCollisionTest.test4_0.bind(null, "simultaneous"));
        schedule(MultipleCollisionTest.test4_0.bind(null, "hybrid"));
        schedule(MultipleCollisionTest.test4_1.bind(null, "serial grouped lastpass"));
        schedule(MultipleCollisionTest.test5_0.bind(null, "simultaneous", MultipleCollisionTest.BALL));
        schedule(MultipleCollisionTest.test5_0.bind(null, "hybrid", MultipleCollisionTest.BALL));
        schedule(MultipleCollisionTest.test5_0.bind(null, "serial grouped lastpass", MultipleCollisionTest.BALL));
        schedule(MultipleCollisionTest.test5_0.bind(null, "simultaneous", MultipleCollisionTest.BLOCK));
        schedule(MultipleCollisionTest.test5_0.bind(null, "hybrid", MultipleCollisionTest.BLOCK));
        schedule(MultipleCollisionTest.test6_0.bind(null, "serial grouped lastpass"));
        schedule(MultipleCollisionTest.test6_1.bind(null, "simultaneous"));
        schedule(MultipleCollisionTest.test6_1.bind(null, "hybrid"));
        schedule(MultipleCollisionTest.test7_0.bind(null, "simultaneous"));
        schedule(MultipleCollisionTest.test7_1.bind(null, "hybrid"));
        schedule(MultipleCollisionTest.test7_2.bind(null, "serial grouped lastpass"));
        schedule(MultipleCollisionTest.test8_0);
        schedule(MultipleCollisionTest.test8_1);
        schedule(MultipleCollisionTest.test8_2);
        schedule(MultipleCollisionTest.test8_3);
        schedule(MultipleCollisionTest.test8_5);
        schedule(MultipleCollisionTest.two_in_box);
        schedule(MultipleCollisionTest.one_hits_two_in_box);
    }
    ;
    static commonSetup1(sim, advance) {
        sim.addForceLaw(new DampingLaw(0, 0.15, sim.getSimList()));
        sim.setCollisionAccuracy(0.6);
        sim.setCollisionHandling("serial grouped lastpass");
        sim.setRandomSeed(99999);
        if (sim instanceof ContactSim) {
            sim.setDistanceTol(MultipleCollisionTest.distanceTol_);
            sim.setVelocityTol(MultipleCollisionTest.velocityTol_);
            sim.setExtraAccel("velocity");
        }
        advance.setJointSmallImpacts(false);
        advance.setTimeStep(0.025);
        advance.setDiffEqSolver(new RungeKutta(sim));
    }
    ;
    static one_hits_wall_setup(sim, advance) {
        MultipleCollisionTest.commonSetup1(sim, advance);
        const b0 = Shapes.makeBlock(1, 1, 'block');
        b0.setPosition(new Vector(0, 2), 0);
        b0.setVelocity(new Vector(0, -3), 0);
        sim.addBody(b0);
        const b1 = Shapes.makeBlock(3, 1, 'wall');
        b1.setPosition(new Vector(0, -2), 0);
        b1.setMass(Infinity);
        sim.addBody(b1);
        sim.setElasticity(1.0);
    }
    ;
    static test1_0(collisionType) {
        setTestName(groupName + 'test1_0 ' + collisionType);
        const sim = new ImpulseSim();
        const advance = new CollisionAdvance(sim);
        MultipleCollisionTest.one_hits_wall_setup(sim, advance);
        sim.setCollisionHandling(collisionType);
        const vars = makeVars(6 * 2);
        setBodyVars(sim, vars, 0, 0, 0, 2.01, 3, 0, 0);
        setBodyVars(sim, vars, 1, 0, 0, -2, 0, 0, 0);
        runTest(sim, advance, 2.0, vars, 0.000001, 0.0, 0.000001, -1);
    }
    ;
    static test1_1(collisionType) {
        setTestName(groupName + 'test1_1 ' + collisionType);
        const sim = new ImpulseSim();
        const advance = new CollisionAdvance(sim);
        MultipleCollisionTest.one_hits_wall_setup(sim, advance);
        sim.setCollisionHandling(collisionType);
        const vars = makeVars(6 * 2);
        setBodyVars(sim, vars, 0, 0, 0, 1.7696, 2.76, -2.8848, -2.88);
        setBodyVars(sim, vars, 1, 0, 0, -2, 0, 0, 0);
        runTest(sim, advance, 2.0, vars, 0.000001, 0.0, 0.000001, -1);
    }
    ;
    static test2_prep(sim, advance, offset, balls) {
        MultipleCollisionTest.commonSetup1(sim, advance);
        const radius = 0.5;
        const b0 = balls ? Shapes.makeBall(radius, 'ball0')
            : Shapes.makeBlock(2 * radius, 2 * radius, 'block0');
        b0.setPosition(new Vector(-3, 0), 0);
        b0.setVelocity(new Vector(3, 0), 0);
        sim.addBody(b0);
        const b1 = balls ? Shapes.makeBall(radius, 'ball1')
            : Shapes.makeBlock(2 * radius, 2 * radius, 'block1');
        b1.setPosition(new Vector(0, 0), 0);
        sim.addBody(b1);
        const b2 = balls ? Shapes.makeBall(radius, 'ball2')
            : Shapes.makeBlock(2 * radius, 2 * radius, 'block2');
        b2.setPosition(new Vector(2 * radius + MultipleCollisionTest.distanceTol_ / 2 + offset, 0), 0);
        sim.addBody(b2);
        sim.setElasticity(1.0);
    }
    ;
    static one_hits_two_ball_setup(sim, advance) {
        MultipleCollisionTest.test2_prep(sim, advance, 0, MultipleCollisionTest.BALL);
    }
    ;
    static one_hits_two_block_setup(sim, advance) {
        MultipleCollisionTest.test2_prep(sim, advance, 0, MultipleCollisionTest.BLOCK);
    }
    ;
    static test2_0(collisionType, balls) {
        setTestName(groupName + 'test2_0 ' + collisionType + ' balls=' + balls);
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        MultipleCollisionTest.test2_prep(sim, advance, 0, balls);
        sim.setCollisionHandling(collisionType);
        const vars = makeVars(6 * 3);
        setBodyVars(sim, vars, 0, -2.34, -1, 0, 0, 0, 0);
        setBodyVars(sim, vars, 1, 2.67, 2, 0, 0, 0, 0);
        setBodyVars(sim, vars, 2, 3.675, 2, 0, 0, 0, 0);
        runTest(sim, advance, 2.0, vars, 0.000001, 0.0, 0.000001, -1);
    }
    ;
    static test2_1(collisionType, balls) {
        setTestName(groupName + 'test2_1 ' + collisionType + ' balls=' + balls);
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        MultipleCollisionTest.test2_prep(sim, advance, 0, balls);
        sim.setCollisionHandling(collisionType);
        const vars = makeVars(6 * 3);
        setBodyVars(sim, vars, 0, -1.005, 0, 0, 0, 0, 0);
        setBodyVars(sim, vars, 1, 0, 0, 0, 0, 0, 0);
        setBodyVars(sim, vars, 2, 5.01, 3, 0, 0, 0, 0);
        runTest(sim, advance, 2.0, vars, 0.000001, 0.0, 0.000001, -1);
    }
    ;
    static two_hits_one_asymmetric_setup(sim, advance) {
        MultipleCollisionTest.commonSetup1(sim, advance);
        const distTol = sim.getDistanceTol();
        const radius = 0.5;
        const b0 = Shapes.makeBall(radius, 'left');
        b0.setPosition(new Vector(-5 + distTol / 2, 0), 0);
        b0.setVelocity(new Vector(3, 0), 0);
        sim.addBody(b0);
        const b1 = Shapes.makeBall(radius, 'center');
        b1.setPosition(new Vector(0, 0), 0);
        sim.addBody(b1);
        const b2 = Shapes.makeBall(radius, 'right');
        b2.setPosition(new Vector(3, 0), 0);
        b2.setVelocity(new Vector(-1.5, 0), 0);
        sim.addBody(b2);
        sim.setElasticity(1.0);
    }
    ;
    static test3_0(collisionType) {
        setTestName(groupName + 'test3_0 ' + collisionType);
        const sim = new ImpulseSim();
        const advance = new CollisionAdvance(sim);
        MultipleCollisionTest.two_hits_one_asymmetric_setup(sim, advance);
        sim.setCollisionHandling(collisionType);
        const vars = makeVars(6 * 3);
        setBodyVars(sim, vars, 0, -2.345, -2, 0, 0, 0, 0);
        setBodyVars(sim, vars, 1, 0.67, 1, 0, 0, 0, 0);
        setBodyVars(sim, vars, 2, 2.68, 2.5, 0, 0, 0, 0);
        runTest(sim, advance, 2.0, vars, 0.000001, 0.0, 0.000001, -1);
    }
    ;
    static test3_1(collisionType) {
        setTestName(groupName + 'test3_1 ' + collisionType);
        const sim = new ImpulseSim();
        const advance = new CollisionAdvance(sim);
        MultipleCollisionTest.two_hits_one_asymmetric_setup(sim, advance);
        sim.setCollisionHandling(collisionType);
        const vars = makeVars(6 * 3);
        setBodyVars(sim, vars, 0, -2.01, -1.5, 0, 0, 0, 0);
        setBodyVars(sim, vars, 1, 0, 0, 0, 0, 0, 0);
        setBodyVars(sim, vars, 2, 3.015, 3, 0, 0, 0, 0);
        runTest(sim, advance, 2.0, vars, 0.000001, 0.0, 0.000001, -1);
    }
    ;
    static one_hits_two_separate_setup(sim, advance) {
        MultipleCollisionTest.commonSetup1(sim, advance);
        const b0 = Shapes.makeBlock(1, 3, 'block0');
        b0.setPosition(new Vector(-4, 0), 0);
        b0.setVelocity(new Vector(3, 0), 0);
        b0.setMass(2);
        sim.addBody(b0);
        const b1 = Shapes.makeBall(0.5, 'ball1');
        b1.setPosition(new Vector(0, 1), 0);
        sim.addBody(b1);
        const b2 = Shapes.makeBall(0.5, 'ball2');
        b2.setPosition(new Vector(0, -1), 0);
        sim.addBody(b2);
        sim.setElasticity(1.0);
    }
    ;
    static test4_0(collisionType) {
        setTestName(groupName + 'test4_0 ' + collisionType);
        const sim = new ImpulseSim();
        const advance = new CollisionAdvance(sim);
        MultipleCollisionTest.one_hits_two_separate_setup(sim, advance);
        sim.setCollisionHandling(collisionType);
        const vars = makeVars(6 * 3);
        setBodyVars(sim, vars, 0, -1.005, 0, 0, 0, 0, 0);
        setBodyVars(sim, vars, 1, 3.005, 3, 1, 0, 0, 0);
        setBodyVars(sim, vars, 2, 3.005, 3, -1, 0, 0, 0);
        runTest(sim, advance, 2.0, vars, 0.000001, 0.0, 0.000001, -1);
    }
    ;
    static test4_1(collisionType) {
        setTestName(groupName + 'test4_1 ' + collisionType);
        const sim = new ImpulseSim();
        const advance = new CollisionAdvance(sim);
        MultipleCollisionTest.one_hits_two_separate_setup(sim, advance);
        sim.setCollisionHandling(collisionType);
        const vars = makeVars(6 * 3);
        setBodyVars(sim, vars, 0, -0.9981859, 0.0068027, 0, 0, 0.1635374, 0.1632653);
        setBodyVars(sim, vars, 1, 3.1344671, 3.1292517, 1, 0, 0, 0);
        setBodyVars(sim, vars, 2, 2.8619048, 2.8571429, -1, 0, 0, 0);
        runTest(sim, advance, 2.0, vars, 0.000001, 0.0, 0.000001, -1);
    }
    ;
    static test5_prep(sim, advance, balls) {
        MultipleCollisionTest.commonSetup1(sim, advance);
        const radius = 0.5;
        const b0 = balls ? Shapes.makeBall(radius, 'ball0')
            : Shapes.makeBlock(2 * radius, 2 * radius, 'block0');
        b0.setPosition(new Vector(-3, 0), 0);
        b0.setVelocity(new Vector(3, 0), 0);
        sim.addBody(b0);
        const b1 = balls ? Shapes.makeBall(radius, 'ball1')
            : Shapes.makeBlock(2 * radius, 2 * radius, 'block1');
        b1.setPosition(new Vector(0, 0), 0);
        sim.addBody(b1);
        const b2 = balls ? Shapes.makeBall(radius, 'ball2')
            : Shapes.makeBlock(2 * radius, 2 * radius, 'block2');
        b2.setPosition(new Vector(2 * radius + MultipleCollisionTest.distanceTol_ / 2, 0), 0);
        sim.addBody(b2);
        const b3 = Shapes.makeBlock(1, 3, 'wall');
        b3.setMass(Infinity);
        b3.setPosition(new Vector(3 * radius + 0.5 + MultipleCollisionTest.distanceTol_, 0), 0);
        sim.addBody(b3);
        sim.setElasticity(1.0);
    }
    ;
    static one_hits_two_on_wall_ball_setup(sim, advance) {
        MultipleCollisionTest.test5_prep(sim, advance, MultipleCollisionTest.BALL);
    }
    ;
    static one_hits_two_on_wall_block_setup(sim, advance) {
        MultipleCollisionTest.test5_prep(sim, advance, MultipleCollisionTest.BLOCK);
    }
    ;
    static test5_0(collisionType, balls) {
        setTestName(groupName + 'test5_0 ' + collisionType + ' balls=' + balls);
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        MultipleCollisionTest.test5_prep(sim, advance, balls);
        sim.setCollisionHandling(collisionType);
        const vars = makeVars(6 * 3);
        setBodyVars(sim, vars, 0, -5.01, -3, 0, 0, 0, 0);
        setBodyVars(sim, vars, 1, 0, 0, 0, 0, 0, 0);
        setBodyVars(sim, vars, 2, 1.005, 0, 0, 0, 0, 0);
        runTest(sim, advance, 2.0, vars, 0.000001, 0.0, 0.000001, -1);
    }
    ;
    static center_spin_setup(sim, advance) {
        MultipleCollisionTest.commonSetup1(sim, advance);
        const b0 = Shapes.makeBlock(1, 3, 'block0');
        b0.setPosition(new Vector(0, 0), 0);
        b0.setVelocity(new Vector(0, 0), 2);
        sim.addBody(b0);
        const b1 = Shapes.makeBlock(1, 3, 'block1');
        b1.setPosition(new Vector(2.8, 0), Math.PI / 2);
        sim.addBody(b1);
        const b2 = Shapes.makeBlock(1, 3, 'block2');
        b2.setPosition(new Vector(-2.8, 0), -Math.PI / 2);
        sim.addBody(b2);
        sim.setElasticity(1.0);
    }
    ;
    static test6_0(collisionType) {
        setTestName(groupName + 'test6_0 ' + collisionType);
        const sim = new ImpulseSim();
        const advance = new CollisionAdvance(sim);
        MultipleCollisionTest.center_spin_setup(sim, advance);
        sim.setCollisionHandling(collisionType);
        const vars = makeVars(6 * 3);
        setBodyVars(sim, vars, 0, 0.1855924, 0.3179762, 0.2040219, 0.3495516, 0.4374322, -0.6771538);
        setBodyVars(sim, vars, 1, 3.0447046, 0.4192534, 0.269004, 0.4608857, 1.2334118, -0.5780423);
        setBodyVars(sim, vars, 2, -3.230297, -0.7372296, -0.473026, -0.8104373, -2.1640649, -1.0164494);
        runTest(sim, advance, 1.0, vars, 0.000001, 0.0, 0.000001, -1);
    }
    ;
    static test6_1(collisionType) {
        setTestName(groupName + 'test6_1 ' + collisionType);
        const sim = new ImpulseSim();
        const advance = new CollisionAdvance(sim);
        MultipleCollisionTest.center_spin_setup(sim, advance);
        sim.setCollisionHandling(collisionType);
        const vars = makeVars(6 * 3);
        setBodyVars(sim, vars, 0, 0, 0, 0, 0, 0.3612164, -0.8077347);
        setBodyVars(sim, vars, 1, 3.1539628, 0.6064458, 0.3891116, 0.6666666, 1.082773, -0.8361323);
        setBodyVars(sim, vars, 2, -3.1539628, -0.6064458, -0.3891116, -0.6666666, -2.0588196, -0.8361323);
        runTest(sim, advance, 1.0, vars, 0.000001, 0.0, 0.000001, -1);
    }
    ;
    static side_spin_setup(sim, advance) {
        MultipleCollisionTest.commonSetup1(sim, advance);
        const b0 = Shapes.makeBlock(1, 3, 'block0');
        b0.setPosition(new Vector(0, 0), Math.PI / 2);
        sim.addBody(b0);
        const b1 = Shapes.makeBlock(1, 3, 'block1');
        b1.setPosition(new Vector(2.8, 1.001), 0);
        b1.setVelocity(new Vector(0, 0), 2);
        sim.addBody(b1);
        const b2 = Shapes.makeBlock(1, 3, 'block2');
        b2.setPosition(new Vector(-2.8, 1.001), Math.PI / 2);
        sim.addBody(b2);
        sim.setElasticity(1.0);
    }
    ;
    static test7_0(collisionType) {
        setTestName(groupName + 'test7_0 ' + collisionType);
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        MultipleCollisionTest.side_spin_setup(sim, advance);
        sim.setCollisionHandling(collisionType);
        sim.setVelocityTol(0.05);
        const vars = makeVars(6 * 3);
        setBodyVars(sim, vars, 0, -0, -0, -0.2407811, -1.1140238, 1.3296854, -1.1155496);
        setBodyVars(sim, vars, 1, 2.8, 0, 1.2018587, 0.9293143, 1.6380862, 0.3255308);
        setBodyVars(sim, vars, 2, -2.8, 0, 1.0409225, 0.1847096, 1.6330754, 0.2881469);
        runTest(sim, advance, 1.0, vars, 0.000001, 0.0, 0.000001, -1);
    }
    ;
    static test7_1(collisionType) {
        setTestName(groupName + 'test7_1 ' + collisionType);
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        MultipleCollisionTest.side_spin_setup(sim, advance);
        sim.setCollisionHandling(collisionType);
        sim.setRandomSeed(86161959);
        const vars = makeVars(6 * 3);
        setBodyVars(sim, vars, 0, -0, -0, -0.2696156, -1.2474324, 1.4082314, -0.7521404);
        setBodyVars(sim, vars, 1, 2.8, 0, 1.1939246, 0.8926056, 1.6523821, 0.3916737);
        setBodyVars(sim, vars, 2, -2.8, 0, 1.077691, 0.3548268, 1.6904343, 0.5535298);
        runTest(sim, advance, 1.0, vars, 0.000001, 0.0, 0.000001, -1);
    }
    ;
    static test7_2(collisionType) {
        setTestName(groupName + 'test7_2 ' + collisionType);
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        MultipleCollisionTest.side_spin_setup(sim, advance);
        sim.setCollisionHandling(collisionType);
        sim.setRandomSeed(86161959);
        const vars = makeVars(6 * 3);
        setBodyVars(sim, vars, 0, -0.0005129, -0.002373, -0.2514185, -1.16324, 1.4220696, -0.6881151);
        setBodyVars(sim, vars, 1, 2.8005129, 0.002373, 1.1681077, 0.7731586, 1.7396203, 0.7952992);
        setBodyVars(sim, vars, 2, -2.8, 0, 1.0853108, 0.3900814, 1.7023212, 0.608527);
        runTest(sim, advance, 1.0, vars, 0.000001, 0.0, 0.000001, -1);
    }
    ;
    static joint_collision_setup(sim, advance) {
        MultipleCollisionTest.commonSetup1(sim, advance);
        const zel = Walls.make(sim, 20, 10);
        const j1 = Shapes.makeBlock(1, 3, 'joint1');
        j1.setPosition(new Vector(2, -5 + 0.5 + 0.005), Math.PI / 2);
        const j2 = Shapes.makeBlock(1, 3, 'joint2');
        j2.setPosition(new Vector(4, -5 + 0.5 + 0.005), Math.PI / 2);
        sim.addBody(j1);
        sim.addBody(j2);
        JointUtil.attachRigidBody(sim, j1, new Vector(0, 1.0), j2, new Vector(0, -1.0), 0);
        sim.alignConnectors();
        const f1 = Shapes.makeBlock(1, 3, 'free1');
        f1.setPosition(new Vector(-6, -5 + 0.5 + 0.005), Math.PI / 2);
        f1.setVelocity(new Vector(3, 0), 0);
        sim.addBody(f1);
        const gravity = new GravityLaw(4.0, sim.getSimList());
        sim.addForceLaw(gravity);
        gravity.setZeroEnergyLevel(zel + 0.5);
        sim.setElasticity(0.8);
    }
    ;
    static test8_0() {
        setTestName(groupName + 'test8_0');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        MultipleCollisionTest.joint_collision_setup(sim, advance);
        sim.setCollisionHandling("serial grouped lastpass");
        sim.setRandomSeed(86161959);
        const vars = makeVars(6 * 7);
        setBodyVars(sim, vars, 4, 5.3468095, 1.3378378, -4.4909991, -0, 1.5708394, -0);
        setBodyVars(sim, vars, 5, 3.3468392, 1.3378378, -4.4943779, 0, 1.5741319, 0);
        setBodyVars(sim, vars, 6, -2.1936486, 0.3243243, -4.49445, 0.0000026, 1.5687089, -0.0000012);
        runTest(sim, advance, 3.5, vars, 0.000001);
        runTest(sim, advance, 5.5, null, 0.000001, 0.0, 0.000001, -1);
    }
    ;
    static joint_collision_2_setup(sim, advance) {
        MultipleCollisionTest.commonSetup1(sim, advance);
        const j1 = Shapes.makeBlock(1, 3, 'joint1');
        j1.setPosition(new Vector(2, 0), Math.PI / 2);
        const j2 = Shapes.makeBlock(1, 3, 'joint2');
        j2.setPosition(new Vector(0, 0), Math.PI / 2);
        j2.setVelocity(new Vector(3, 0), 0);
        sim.addBody(j1);
        sim.addBody(j2);
        JointUtil.attachRigidBody(sim, j1, new Vector(0, 1.0), j2, new Vector(0, -1.0), 0);
        sim.alignConnectors();
        const collisions = [];
        sim.findCollisions(collisions, sim.getVarsList().getValues(), 0);
        sim.handleCollisions(collisions);
        sim.setElasticity(0.8);
    }
    ;
    static test8_1() {
        setTestName(groupName + 'test8_1');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        MultipleCollisionTest.joint_collision_2_setup(sim, advance);
        sim.setCollisionHandling("serial grouped lastpass");
        sim.setRandomSeed(86161959);
        const vars = makeVars(6 * 2);
        setBodyVars(sim, vars, 0, 6.5, 1.5, -0, -0, 1.5707963, 0);
        setBodyVars(sim, vars, 1, 4.5, 1.5, 0, 0, 1.5707963, 0);
        runTest(sim, advance, 3.0, vars, 0.000001);
    }
    ;
    static joint_collision_3_setup(sim, advance) {
        MultipleCollisionTest.commonSetup1(sim, advance);
        const j1 = Shapes.makeBlock(1, 3, 'joint1');
        j1.setPosition(new Vector(2, 0), Math.PI / 2);
        sim.addBody(j1);
        const j2 = Shapes.makeBlock(1, 3, 'joint2');
        j2.setPosition(new Vector(0, 0), Math.PI / 2);
        sim.addBody(j2);
        JointUtil.attachRigidBody(sim, j1, new Vector(0, 1.0), j2, new Vector(0, -1.0), 0);
        sim.alignConnectors();
        const f1 = Shapes.makeBall(0.5, 'free1');
        f1.setPosition(new Vector(-6, 0), Math.PI / 2);
        f1.setVelocity(new Vector(3, 0), 0);
        sim.addBody(f1);
        sim.setElasticity(1.0);
    }
    ;
    static test8_2() {
        setTestName(groupName + 'test8_2');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        MultipleCollisionTest.joint_collision_3_setup(sim, advance);
        sim.setCollisionHandling("serial grouped lastpass");
        sim.setRandomSeed(86161959);
        const vars = makeVars(6 * 3);
        setBodyVars(sim, vars, 0, 5.3366667, 2, -0, -0, 1.5707963, 0);
        setBodyVars(sim, vars, 1, 3.3366667, 2, -0, -0, 1.5707963, 0);
        setBodyVars(sim, vars, 2, -3.6733333, -1, 0, 0, 1.5707963, 0);
        runTest(sim, advance, 3.0, vars, 0.000001);
    }
    ;
    static joint_collision_4_setup(sim, advance) {
        MultipleCollisionTest.commonSetup1(sim, advance);
        const j1 = Shapes.makeBlock(1, 3, 'joint1');
        j1.setPosition(new Vector(2, 0), Math.PI / 2);
        j1.setVelocity(new Vector(0, 0), 3);
        sim.addBody(j1);
        const j2 = Shapes.makeBlock(1, 3, 'joint2');
        j2.setPosition(new Vector(0, 0), Math.PI / 2);
        j2.setVelocity(new Vector(0, 0), -5);
        sim.addBody(j2);
        JointUtil.attachRigidBody(sim, j1, new Vector(0, 1.0), j2, new Vector(0, -1.0), 0);
        sim.alignConnectors();
        sim.setElasticity(0.8);
        const collisions = [];
        sim.findCollisions(collisions, sim.getVarsList().getValues(), 0);
        sim.handleCollisions(collisions);
    }
    ;
    static test8_3() {
        setTestName(groupName + 'test8_3');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        MultipleCollisionTest.joint_collision_4_setup(sim, advance);
        sim.setCollisionHandling("serial grouped lastpass");
        sim.setRandomSeed(86161959);
        const vars = makeVars(6 * 2);
        setBodyVars(sim, vars, 0, 1.4983143, 0.2052494, 0.8618191, -0.5534606, 8.9952832, 3.5216341);
        setBodyVars(sim, vars, 1, 0.5016857, -0.2052494, -0.8618191, 0.5534606, -10.0437751, -4.435186);
        runTest(sim, advance, 3.0, vars, 0.000001);
    }
    ;
    static joint_collision_5_setup(sim, advance) {
        MultipleCollisionTest.commonSetup1(sim, advance);
        const distTol = sim.getDistanceTol();
        let body = Shapes.makeBall(0.5, 'body4');
        body.setMass(2);
        body.setPosition(new Vector(1 + distTol / 2, 0));
        sim.addBody(body);
        body = Shapes.makeBall(0.5, 'body1');
        body.setMass(2);
        body.setPosition(new Vector(-5, 0), 0);
        body.setVelocity(new Vector(3, 0), 0);
        sim.addBody(body);
        const body2 = Shapes.makeBlock(2, 1, 'body2');
        body2.setPosition(new Vector(-2, 0));
        sim.addBody(body2);
        const body3 = Shapes.makeBlock(2, 1, 'body3');
        body3.setPosition(new Vector(-0.5, 0));
        sim.addBody(body3);
        JointUtil.attachRigidBody(sim, body2, new Vector(0.75, 0), body3, new Vector(-0.75, 0), 0);
        sim.alignConnectors();
        sim.setElasticity(1.0);
    }
    ;
    static test8_5() {
        setTestName(groupName + 'test8_5');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        MultipleCollisionTest.joint_collision_5_setup(sim, advance);
        sim.setCollisionHandling("serial grouped lastpass");
        sim.setRandomSeed(86161959);
        const vars = makeVars(6 * 4);
        setBodyVars(sim, vars, 0, 2.51, 3, 0, 0, 0, 0);
        setBodyVars(sim, vars, 1, -3.505, 0, 0, 0, 0, 0);
        setBodyVars(sim, vars, 2, -2, 0, 0, 0, 0, 0);
        setBodyVars(sim, vars, 3, -0.5, 0, 0, 0, 0, 0);
        runTest(sim, advance, 1.0, vars, 0.000001);
    }
    ;
    static two_in_box_setup(sim, advance) {
        MultipleCollisionTest.commonSetup1(sim, advance);
        const distTol = sim.getDistanceTol();
        let body = Shapes.makeFrame(2 + 3 * distTol / 2 + 0.2, 1 + 2 * distTol / 2 + 0.2, 0.2, 'body1');
        body.setPosition(new Vector(0, 0));
        sim.addBody(body);
        body = Shapes.makeBall(0.5, 'body2');
        body.setPosition(new Vector(-0.5 - distTol / 4, 0));
        sim.addBody(body);
        body = Shapes.makeBall(0.5, 'body3');
        body.setPosition(new Vector(0.5 + distTol / 4, 0));
        body.setVelocity(new Vector(3, 0), 0);
        sim.addBody(body);
        sim.setElasticity(1.0);
    }
    ;
    static two_in_box() {
        setTestName(groupName + 'two_in_box');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        MultipleCollisionTest.two_in_box_setup(sim, advance);
        sim.setCollisionHandling("serial grouped lastpass");
        sim.setRandomSeed(86161959);
        const vars = makeVars(6 * 3);
        setBodyVars(sim, vars, 0, 1, 1, 0, 0, 0, 0);
        setBodyVars(sim, vars, 1, 0.4975, 1, 0, 0, 0, 0);
        setBodyVars(sim, vars, 2, 1.5025, 1, 0, 0, 0, 0);
        runTest(sim, advance, 1.0, vars, 0.000001);
    }
    ;
    static one_hits_two_in_box_setup(sim, advance) {
        MultipleCollisionTest.commonSetup1(sim, advance);
        const distTol = sim.getDistanceTol();
        let body = Shapes.makeFrame(2 + 3 * distTol / 2 + 0.2, 1 + 2 * distTol / 2 + 0.2, 0.2, 'body1');
        body.setPosition(new Vector(0, 0));
        sim.addBody(body);
        body = Shapes.makeBall(0.5, 'body2');
        body.setPosition(new Vector(-0.5 - distTol / 4, 0));
        sim.addBody(body);
        body = Shapes.makeBall(0.5, 'body3');
        body.setPosition(new Vector(0.5 + distTol / 4, 0));
        sim.addBody(body);
        body = Shapes.makeBall(0.5, 'body4');
        body.setPosition(new Vector(-5, 0));
        body.setVelocity(new Vector(3, 0), 0);
        sim.addBody(body);
        sim.setElasticity(1.0);
    }
    ;
    static one_hits_two_in_box() {
        setTestName(groupName + 'two_in_box');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        MultipleCollisionTest.one_hits_two_in_box_setup(sim, advance);
        sim.setCollisionHandling("serial grouped lastpass");
        sim.setRandomSeed(86161959);
        const vars = makeVars(6 * 4);
        setBodyVars(sim, vars, 0, 0.9041667, 1, 0, 0, 0, 0);
        setBodyVars(sim, vars, 1, 0.4016667, 1, 0, 0, 0, 0);
        setBodyVars(sim, vars, 2, 1.4066667, 1, 0, 0, 0, 0);
        setBodyVars(sim, vars, 3, -1.7125, 0, 0, 0, 0, 0);
        runTest(sim, advance, 2.0, vars, 0.000001);
    }
    ;
}
MultipleCollisionTest.BALL = true;
MultipleCollisionTest.BLOCK = false;
MultipleCollisionTest.distanceTol_ = 0.01;
MultipleCollisionTest.velocityTol_ = 0.5;
