import { CollisionAdvance } from '../lab/model/CollisionAdvance.js';
import { ContactSim } from '../lab/engine2D/ContactSim.js';
import { DampingLaw } from '../lab/model/DampingLaw.js';
import { DoubleRect } from '../lab/util/DoubleRect.js';
import { ImpulseSim } from '../lab/engine2D/ImpulseSim.js';
import { ModifiedEuler } from '../lab/model/ModifiedEuler.js';
import { RungeKutta } from '../lab/model/RungeKutta.js';
import { Shapes } from '../lab/engine2D/Shapes.js';
import { Util } from '../lab/util/Util.js';
import { Vector } from '../lab/util/Vector.js';
import { Walls } from '../lab/engine2D/Walls.js';
import { schedule, setTestName } from './TestRig.js';
import { makeVars, setBodyVars, runTest } from './Engine2DTestRig.js';
const groupName = 'SpeedTest.';
export class SpeedTest {
    constructor() { throw ''; }
    ;
    static test() {
        schedule(SpeedTest.ball_vs_wall_0);
        schedule(SpeedTest.ball_vs_wall_1);
        schedule(SpeedTest.ball_vs_circle_0);
        schedule(SpeedTest.small_small);
    }
    ;
    static commonSetup1(sim, advance, damping = 0.05) {
        sim.addForceLaw(new DampingLaw(damping, 0.15, sim.getSimList()));
        if (sim instanceof ContactSim) {
            sim.setDistanceTol(0.01);
            sim.setVelocityTol(0.5);
            sim.setExtraAccel("velocity_and_distance");
        }
        sim.setCollisionAccuracy(0.6);
        sim.setCollisionHandling("serial grouped lastpass");
        advance.setDiffEqSolver(new RungeKutta(sim));
        advance.setJointSmallImpacts(true);
        advance.setTimeStep(0.025);
    }
    ;
    static ball_vs_wall_setup(sim, advance) {
        SpeedTest.commonSetup1(sim, advance);
        const p = Shapes.makeBall(0.2, 'fast_ball');
        p.setMass(0.1);
        p.setPosition(new Vector(-5, 0), 0);
        p.setVelocity(new Vector(200, 153), 0);
        p.setElasticity(0.95);
        sim.addBody(p);
        Walls.make(sim, 12.0, 12.0, 0.01);
    }
    ;
    static ball_vs_wall_0() {
        setTestName(groupName + 'ball_vs_wall_0');
        const sim = new ImpulseSim();
        const advance = new CollisionAdvance(sim);
        SpeedTest.ball_vs_wall_setup(sim, advance);
        const vars = makeVars(6 * 1);
        setBodyVars(sim, vars, 0, -4.349276, 0.4830918, 4.4247788, -0.4310427, 0, 0);
        runTest(sim, advance, 10.0, vars, 0.00001, NaN, NaN, 37);
    }
    ;
    static ball_vs_wall_1() {
        setTestName(groupName + 'ball_vs_wall_1');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        SpeedTest.ball_vs_wall_setup(sim, advance);
        const vars = makeVars(6 * 1);
        setBodyVars(sim, vars, 0, -4.3493183, 0.4830918, 4.4248186, -0.4310427, 0, 0);
        runTest(sim, advance, 10.0, vars, 0.00001, NaN, NaN, 37);
    }
    ;
    static ball_vs_circle_setup(sim, advance) {
        SpeedTest.commonSetup1(sim, advance);
        const p = Shapes.makeBall(0.2, 'fast_ball');
        p.setMass(0.1);
        p.setPosition(new Vector(-5, 0), 0);
        p.setVelocity(new Vector(200, 153), 0);
        sim.addBody(p);
        Walls.make(sim, 12.0, 12.0, 0.01);
        const b = Shapes.makeBall(4, 'fixBall');
        b.setMass(Infinity);
        b.setPosition(new Vector(0, 0), 0);
        sim.addBody(b);
        sim.setElasticity(0.95);
    }
    ;
    static ball_vs_circle_0() {
        setTestName(groupName + 'ball_vs_circle_0');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        SpeedTest.ball_vs_circle_setup(sim, advance);
        const vars = makeVars(6 * 1);
        let expectedCollisions;
        if (Util.isChrome()) {
            setBodyVars(sim, vars, 0, -5.6805659, -0.1123115, 5.6022192, 0.2347565, -0.0000012, -0);
            expectedCollisions = 55;
        }
        else {
            setBodyVars(sim, vars, 0, 4.2864427, 0.1090598, 5.7947777, -0.0042925, -0.0000012, -0);
            expectedCollisions = 52;
        }
        runTest(sim, advance, 10, vars, 0.00001, NaN, 0, expectedCollisions, 6);
    }
    ;
    static small_small_setup(sim, advance) {
        SpeedTest.commonSetup1(sim, advance, 0);
        sim.setSimRect(new DoubleRect(-32, -12, 32, 12));
        advance.setDiffEqSolver(new ModifiedEuler(sim));
        advance.setTimeStep(0.1);
        const b1 = Shapes.makeBlock(1, 1, 'block0');
        b1.setPosition(new Vector(-30, 0), Math.PI / 4);
        b1.setVelocity(new Vector(100, 0), 0);
        const b2 = Shapes.makeBlock(1, 1.3, 'block1');
        b2.setPosition(new Vector(5, 0));
        sim.addBody(b1);
        sim.addBody(b2);
        sim.setElasticity(1.0);
    }
    ;
    static small_small() {
        setTestName(groupName + 'small_small');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        SpeedTest.small_small_setup(sim, advance);
        const vars = makeVars(6 * 2);
        setBodyVars(sim, vars, 0, 3.7878932, 0, 0, 0, 0.7853982, -0);
        setBodyVars(sim, vars, 1, 71.2121068, 100, 0, 0, 0, 0);
        runTest(sim, advance, 1.0, vars, 0.00001, 0.0, 0.00001, 1);
    }
    ;
}
